#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.bosnia_and_herzegovina import BosniaAndHerzegovina, BA, BIH
from tests.common import CommonCountryTests


class TestBosniaAndHerzegovina(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(2000, 2030)
        super().setUpClass(BosniaAndHerzegovina, years=years)
        cls.bih_holidays = BosniaAndHerzegovina(subdiv="BIH", years=years)
        cls.brc_holidays = BosniaAndHerzegovina(subdiv="BRC", years=years)
        cls.srp_holidays = BosniaAndHerzegovina(subdiv="SRP", years=years)
        cls.bih_holidays_non_obs = BosniaAndHerzegovina(subdiv="BIH", observed=False, years=years)
        cls.brc_holidays_non_obs = BosniaAndHerzegovina(subdiv="BRC", observed=False, years=years)
        cls.srp_holidays_non_obs = BosniaAndHerzegovina(subdiv="SRP", observed=False, years=years)

    def test_country_aliases(self):
        self.assertAliases(BosniaAndHerzegovina, BA, BIH)

    def test_new_years(self):
        name = "Nova godina"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(2000, 2030)))
        self.assertHolidayName(name, (f"{year}-01-02" for year in range(2000, 2030)))

        dt = (
            "2006-01-03",
            "2012-01-03",
            "2017-01-03",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (preneseno)", self.bih_holidays, dt)
        self.assertNoNonObservedHoliday(self.bih_holidays_non_obs, dt)

        dt = (
            "2000-01-03",
            "2005-01-03",
            "2006-01-03",
            "2011-01-03",
            "2012-01-03",
            "2017-01-03",
            "2022-01-03",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (preneseno)", self.brc_holidays, dt)
        self.assertNoNonObservedHoliday(self.brc_holidays_non_obs, dt)

        dt = (
            "2000-01-03",
            "2005-01-03",
            "2011-01-03",
            "2022-01-03",
        )
        self.assertHolidayName(f"{name} (preneseno)", self.srp_holidays, dt)
        self.assertNoNonObservedHoliday(self.srp_holidays_non_obs, dt)

    def test_orthodox_christmas_eve(self):
        name = "Badnji dan (Pravoslavni)"
        self.assertHolidayName(
            name, self.bih_holidays, (f"{year}-01-06" for year in range(2000, 2030))
        )
        self.assertHolidayName(
            name, self.srp_holidays, (f"{year}-01-06" for year in range(2000, 2030))
        )
        self.assertNoHolidayName(name, self.brc_holidays)
        self.assertNoHolidayName(name)

    def test_orthodox_christmas(self):
        name = "Božić (Pravoslavni)"
        self.assertHolidayName(name, (f"{year}-01-07" for year in range(2000, 2030)))

        dt = (
            "2001-01-08",
            "2007-01-08",
            "2018-01-08",
        )
        self.assertHolidayName(f"{name} (preneseno)", self.brc_holidays, dt)
        self.assertNoNonObservedHoliday(self.brc_holidays_non_obs, dt)
        self.assertNoHoliday(dt)

    def test_orthodox_new_year(self):
        name = "Pravoslavna Nova godina"
        self.assertHolidayName(
            name, self.srp_holidays, (f"{year}-01-14" for year in range(2000, 2030))
        )
        self.assertNoHolidayName(name, self.brc_holidays)
        self.assertNoHolidayName(name, self.bih_holidays)
        self.assertNoHolidayName(name)

    def test_independence_day(self):
        name = "Dan nezavisnosti"
        self.assertHolidayName(
            name, self.bih_holidays, (f"{year}-03-01" for year in range(2000, 2030))
        )
        self.assertNoHolidayName(name, self.brc_holidays)
        self.assertNoHolidayName(name, self.srp_holidays)
        self.assertNoHolidayName(name)

    def test_establishment_bd_day(self):
        name = "Dan uspostavljanja Brčko distrikta"
        self.assertHolidayName(
            name, self.brc_holidays, (f"{year}-03-08" for year in range(2000, 2030))
        )
        self.assertNoHolidayName(name, self.bih_holidays)
        self.assertNoHolidayName(name, self.srp_holidays)
        self.assertNoHolidayName(name)

        dt = (
            "2009-03-09",
            "2015-03-09",
            "2020-03-09",
        )
        self.assertHolidayName(f"{name} (preneseno)", self.brc_holidays, dt)
        self.assertNoNonObservedHoliday(self.brc_holidays_non_obs, dt)
        self.assertNoHoliday(dt)

    def test_catholic_easter(self):
        name = "Veliki petak (Katolički)"
        dt = (
            "2012-04-06",
            "2015-04-03",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )
        self.assertHolidayName(name, self.bih_holidays, dt)
        self.assertHolidayName(name, self.srp_holidays, dt)
        self.assertNoHolidayName(name, self.brc_holidays)
        self.assertNoHolidayName(name)

        name = "Uskrs (Katolički)"
        dt = (
            "2012-04-08",
            "2015-04-05",
            "2018-04-01",
            "2019-04-21",
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
        )
        self.assertHolidayName(name, self.bih_holidays, dt)
        self.assertHolidayName(name, self.srp_holidays, dt)
        self.assertNoHolidayName(name, self.brc_holidays)
        self.assertNoHolidayName(name)

        name = "Uskrsni ponedjeljak (Katolički)"
        dt = (
            "2012-04-09",
            "2015-04-06",
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
        )
        self.assertHolidayName(name, self.bih_holidays, dt)
        self.assertHolidayName(name, self.srp_holidays, dt)
        self.assertHolidayName(name, self.brc_holidays, dt)
        self.assertHolidayName(name, dt)

    def test_orthodox_easter(self):
        name = "Veliki petak (Pravoslavni)"
        dt = (
            "2012-04-13",
            "2015-04-10",
            "2018-04-06",
            "2019-04-26",
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
        )
        self.assertHolidayName(name, self.bih_holidays, dt)
        self.assertHolidayName(name, self.srp_holidays, dt)
        self.assertHolidayName(name, self.brc_holidays, dt)
        self.assertHolidayName(name, dt)

        name = "Vaskrs (Pravoslavni)"
        dt = (
            "2012-04-15",
            "2015-04-12",
            "2018-04-08",
            "2019-04-28",
            "2020-04-19",
            "2022-04-24",
            "2023-04-16",
        )
        self.assertHolidayName(name, self.bih_holidays, dt)
        self.assertHolidayName(name, self.srp_holidays, dt)

        name = "Uskrsni ponedjeljak (Pravoslavni)"
        dt = (
            "2012-04-16",
            "2015-04-13",
            "2018-04-09",
            "2019-04-29",
            "2020-04-20",
            "2021-05-03",
            "2022-04-25",
            "2023-04-17",
        )
        self.assertHolidayName(name, self.bih_holidays, dt)
        self.assertHolidayName(name, self.srp_holidays, dt)
        self.assertNoHolidayName(name, self.brc_holidays)
        self.assertNoHolidayName(name)

    def test_labor_day(self):
        name = "Međunarodni praznik rada"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(2000, 2030)))
        self.assertHolidayName(name, (f"{year}-05-02" for year in range(2000, 2030)))

        dt = (
            "2011-05-03",
            "2022-05-03",
        )
        self.assertHolidayName(f"{name} (preneseno)", self.bih_holidays, dt)
        self.assertNoNonObservedHolidayName(f"{name} (preneseno)", self.bih_holidays_non_obs, dt)

        dt = (
            "2004-05-03",
            "2005-05-03",
            "2010-05-03",
            "2011-05-03",
            "2016-05-03",
            "2021-05-03",
            "2022-05-03",
        )
        self.assertHolidayName(f"{name} (preneseno)", self.brc_holidays, dt)
        self.assertNoNonObservedHoliday(self.brc_holidays_non_obs, dt)

        dt = (
            "2004-05-03",
            "2010-05-03",
            "2021-05-03",
        )
        self.assertHolidayName(f"{name} (preneseno)", self.srp_holidays, dt)
        self.assertNoNonObservedHolidayName(f"{name} (preneseno)", self.srp_holidays_non_obs, dt)

    def test_victory_day(self):
        name = "Dan pobjede nad fašizmom"
        self.assertHolidayName(
            name, self.bih_holidays, (f"{year}-05-09" for year in range(2000, 2030))
        )
        self.assertHolidayName(
            name, self.srp_holidays, (f"{year}-05-09" for year in range(2000, 2030))
        )
        self.assertNoHolidayName(name, self.brc_holidays)
        self.assertNoHolidayName(name)

    def test_eid_al_fitr(self):
        name = "Ramazanski Bajram"
        dt = (
            "2010-09-10",
            "2015-07-18",
            "2018-06-15",
            "2019-06-04",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, self.bih_holidays, dt)
        self.assertHolidayName(name, self.brc_holidays, dt)
        self.assertHolidayName(name, self.srp_holidays, dt)

        dt = (
            "2010-09-11",
            "2015-07-19",
            "2018-06-16",
            "2019-06-05",
            "2020-05-25",
            "2021-05-14",
            "2022-05-03",
            "2023-04-22",
        )
        self.assertHolidayName(name, self.bih_holidays, dt)
        self.assertHolidayName(name, self.srp_holidays, dt)
        self.assertNoHolidayName(name, self.brc_holidays, dt)
        self.assertNoHolidayName(name, dt)

    def test_eid_al_adha(self):
        name = "Kurban Bajram"
        dt = (
            "2006-01-10",
            "2006-12-31",
            "2010-11-17",
            "2015-09-24",
            "2018-08-22",
            "2019-08-11",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, self.bih_holidays, dt)
        self.assertHolidayName(name, self.brc_holidays, dt)
        self.assertHolidayName(name, self.srp_holidays, dt)

        dt = (
            "2007-01-01",
            "2010-11-18",
            "2015-09-25",
            "2018-08-23",
            "2019-08-12",
            "2020-08-01",
            "2021-07-21",
            "2022-07-10",
            "2023-06-29",
        )
        self.assertHolidayName(name, self.bih_holidays, dt)
        self.assertHolidayName(name, self.srp_holidays, dt)
        self.assertNoHolidayName(name, self.brc_holidays, dt)
        self.assertNoHolidayName(name, dt)

    def test_dayton_agreement_day(self):
        name = "Dan uspostave Opšteg okvirnog sporazuma za mir u Bosni i Hercegovini"
        self.assertHolidayName(
            name, self.srp_holidays, (f"{year}-11-21" for year in range(2000, 2030))
        )
        self.assertNoHolidayName(name, self.bih_holidays)
        self.assertNoHolidayName(name, self.brc_holidays)
        self.assertNoHolidayName(name)

    def test_statehood_day(self):
        name = "Dan državnosti"
        self.assertHolidayName(
            name, self.bih_holidays, (f"{year}-11-25" for year in range(2004, 2030))
        )
        self.assertNoHolidayName(name, self.brc_holidays)
        self.assertNoHolidayName(name, self.srp_holidays)
        self.assertNoHolidayName(name)

    def test_catholic_christmas(self):
        name = "Božić (Katolički)"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(2000, 2030)))

        dt = (
            "2005-12-26",
            "2011-12-26",
            "2016-12-26",
        )
        self.assertHolidayName(f"{name} (preneseno)", self.brc_holidays, dt)
        self.assertNoNonObservedHoliday(self.brc_holidays_non_obs, dt)
        self.assertNoHoliday(dt)

        name = "Badnji dan (Katolički)"
        self.assertHolidayName(
            name, self.bih_holidays, (f"{year}-12-24" for year in range(2000, 2030))
        )
        self.assertHolidayName(
            name, self.srp_holidays, (f"{year}-12-24" for year in range(2000, 2030))
        )
        self.assertNoHolidayName(name, self.brc_holidays)
        self.assertNoHolidayName(name)

    def test_2021(self):
        self.assertHolidays(
            BosniaAndHerzegovina(years=2021),
            ("2021-01-01", "Nova godina"),
            ("2021-01-02", "Nova godina"),
            ("2021-01-07", "Božić (Pravoslavni)"),
            ("2021-04-05", "Uskrsni ponedjeljak (Katolički)"),
            ("2021-04-30", "Veliki petak (Pravoslavni)"),
            ("2021-05-01", "Međunarodni praznik rada"),
            ("2021-05-02", "Međunarodni praznik rada"),
            ("2021-05-13", "Ramazanski Bajram"),
            ("2021-07-20", "Kurban Bajram"),
            ("2021-12-25", "Božić (Katolički)"),
        )

        self.assertHolidays(
            BosniaAndHerzegovina(subdiv="BIH", years=2021),
            ("2021-01-01", "Nova godina"),
            ("2021-01-02", "Nova godina"),
            ("2021-01-06", "Badnji dan (Pravoslavni)"),
            ("2021-01-07", "Božić (Pravoslavni)"),
            ("2021-03-01", "Dan nezavisnosti"),
            ("2021-04-02", "Veliki petak (Katolički)"),
            ("2021-04-04", "Uskrs (Katolički)"),
            ("2021-04-05", "Uskrsni ponedjeljak (Katolički)"),
            ("2021-04-30", "Veliki petak (Pravoslavni)"),
            ("2021-05-01", "Međunarodni praznik rada"),
            ("2021-05-02", "Međunarodni praznik rada; Vaskrs (Pravoslavni)"),
            ("2021-05-03", "Uskrsni ponedjeljak (Pravoslavni)"),
            ("2021-05-09", "Dan pobjede nad fašizmom"),
            ("2021-05-13", "Ramazanski Bajram"),
            ("2021-05-14", "Ramazanski Bajram"),
            ("2021-07-20", "Kurban Bajram"),
            ("2021-07-21", "Kurban Bajram"),
            ("2021-11-25", "Dan državnosti"),
            ("2021-12-24", "Badnji dan (Katolički)"),
            ("2021-12-25", "Božić (Katolički)"),
        )

        self.assertHolidays(
            BosniaAndHerzegovina(subdiv="BRC", years=2021),
            ("2021-01-01", "Nova godina"),
            ("2021-01-02", "Nova godina"),
            ("2021-01-07", "Božić (Pravoslavni)"),
            ("2021-03-08", "Dan uspostavljanja Brčko distrikta"),
            ("2021-04-05", "Uskrsni ponedjeljak (Katolički)"),
            ("2021-04-30", "Veliki petak (Pravoslavni)"),
            ("2021-05-01", "Međunarodni praznik rada"),
            ("2021-05-02", "Međunarodni praznik rada"),
            ("2021-05-03", "Međunarodni praznik rada (preneseno)"),
            ("2021-05-13", "Ramazanski Bajram"),
            ("2021-07-20", "Kurban Bajram"),
            ("2021-12-25", "Božić (Katolički)"),
        )

        self.assertHolidays(
            BosniaAndHerzegovina(subdiv="SRP", years=2021),
            ("2021-01-01", "Nova godina"),
            ("2021-01-02", "Nova godina"),
            ("2021-01-06", "Badnji dan (Pravoslavni)"),
            ("2021-01-07", "Božić (Pravoslavni)"),
            ("2021-01-14", "Pravoslavna Nova godina"),
            ("2021-04-02", "Veliki petak (Katolički)"),
            ("2021-04-04", "Uskrs (Katolički)"),
            ("2021-04-05", "Uskrsni ponedjeljak (Katolički)"),
            ("2021-04-30", "Veliki petak (Pravoslavni)"),
            ("2021-05-01", "Međunarodni praznik rada"),
            ("2021-05-02", "Međunarodni praznik rada; Vaskrs (Pravoslavni)"),
            (
                "2021-05-03",
                "Međunarodni praznik rada (preneseno); Uskrsni ponedjeljak (Pravoslavni)",
            ),
            ("2021-05-09", "Dan pobjede nad fašizmom"),
            ("2021-05-13", "Ramazanski Bajram"),
            ("2021-05-14", "Ramazanski Bajram"),
            ("2021-07-20", "Kurban Bajram"),
            ("2021-07-21", "Kurban Bajram"),
            ("2021-11-21", "Dan uspostave Opšteg okvirnog sporazuma za mir u Bosni i Hercegovini"),
            ("2021-12-24", "Badnji dan (Katolički)"),
            ("2021-12-25", "Božić (Katolički)"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Nova godina"),
            ("2022-01-02", "Nova godina"),
            ("2022-01-03", "Nova godina (preneseno)"),
            ("2022-01-06", "Badnji dan (Pravoslavni)"),
            ("2022-01-07", "Božić (Pravoslavni)"),
            ("2022-01-14", "Pravoslavna Nova godina"),
            ("2022-03-01", "Dan nezavisnosti"),
            ("2022-03-08", "Dan uspostavljanja Brčko distrikta"),
            ("2022-04-15", "Veliki petak (Katolički)"),
            ("2022-04-17", "Uskrs (Katolički)"),
            ("2022-04-18", "Uskrsni ponedjeljak (Katolički)"),
            ("2022-04-22", "Veliki petak (Pravoslavni)"),
            ("2022-04-24", "Vaskrs (Pravoslavni)"),
            ("2022-04-25", "Uskrsni ponedjeljak (Pravoslavni)"),
            ("2022-05-01", "Međunarodni praznik rada"),
            ("2022-05-02", "Međunarodni praznik rada; Ramazanski Bajram"),
            ("2022-05-03", "Međunarodni praznik rada (preneseno); Ramazanski Bajram"),
            ("2022-05-09", "Dan pobjede nad fašizmom"),
            ("2022-07-09", "Kurban Bajram"),
            ("2022-07-10", "Kurban Bajram"),
            ("2022-11-21", "Dan uspostave Opšteg okvirnog sporazuma za mir u Bosni i Hercegovini"),
            ("2022-11-25", "Dan državnosti"),
            ("2022-12-24", "Badnji dan (Katolički)"),
            ("2022-12-25", "Božić (Katolički)"),
            ("2022-12-26", "Božić (Katolički) (preneseno)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-02", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-06", "Orthodox Christmas Eve"),
            ("2022-01-07", "Orthodox Christmas Day"),
            ("2022-01-14", "Orthodox New Year"),
            ("2022-03-01", "Independence Day"),
            ("2022-03-08", "Day of establishment of Brčko District"),
            ("2022-04-15", "Catholic Good Friday"),
            ("2022-04-17", "Catholic Easter Sunday"),
            ("2022-04-18", "Catholic Easter Monday"),
            ("2022-04-22", "Orthodox Good Friday"),
            ("2022-04-24", "Orthodox Easter Sunday"),
            ("2022-04-25", "Orthodox Easter Monday"),
            ("2022-05-01", "International Labor Day"),
            ("2022-05-02", "Eid al-Fitr; International Labor Day"),
            ("2022-05-03", "Eid al-Fitr; International Labor Day (observed)"),
            ("2022-05-09", "Victory Day"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-07-10", "Eid al-Adha"),
            ("2022-11-21", "Dayton Agreement Day"),
            ("2022-11-25", "Statehood Day"),
            ("2022-12-24", "Catholic Christmas Eve"),
            ("2022-12-25", "Catholic Christmas Day"),
            ("2022-12-26", "Catholic Christmas Day (observed)"),
        )

    def test_l10n_sr(self):
        self.assertLocalizedHolidays(
            "sr",
            ("2022-01-01", "Нова година"),
            ("2022-01-02", "Нова година"),
            ("2022-01-03", "Нова година (пренешено)"),
            ("2022-01-06", "Бадњи дан (Православни)"),
            ("2022-01-07", "Божић (Православни)"),
            ("2022-01-14", "Православна Нова година"),
            ("2022-03-01", "Дан независности"),
            ("2022-03-08", "Дан успостављања Брчко дистрикта"),
            ("2022-04-15", "Велики петак (Католички)"),
            ("2022-04-17", "Ускрс (Католички)"),
            ("2022-04-18", "Ускршњи понедељак (Католички)"),
            ("2022-04-22", "Велики петак (Православни)"),
            ("2022-04-24", "Васкрс (Православни)"),
            ("2022-04-25", "Ускршњи понедељак (Православни)"),
            ("2022-05-01", "Међународни празник рада"),
            ("2022-05-02", "Међународни празник рада; Рамазански Бајрам"),
            ("2022-05-03", "Међународни празник рада (пренешено); Рамазански Бајрам"),
            ("2022-05-09", "Дан побједе над фашизмом"),
            ("2022-07-09", "Курбан Бајрам"),
            ("2022-07-10", "Курбан Бајрам"),
            ("2022-11-21", "Дан успоставе Општег оквирног споразума за мир у Босни и Херцеговини"),
            ("2022-11-25", "Дан државности"),
            ("2022-12-24", "Бадњи дан (Католички)"),
            ("2022-12-25", "Божић (Католички)"),
            ("2022-12-26", "Божић (Католички) (пренешено)"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-02", "Новий рік"),
            ("2022-01-03", "Новий рік (вихідний)"),
            ("2022-01-06", "Святий вечір (православний)"),
            ("2022-01-07", "Різдво Христове (православне)"),
            ("2022-01-14", "Православний Новий рік"),
            ("2022-03-01", "День незалежності"),
            ("2022-03-08", "День заснування округу Брчко"),
            ("2022-04-15", "Страсна пʼятниця (католицька)"),
            ("2022-04-17", "Великдень (католицький)"),
            ("2022-04-18", "Великодній понеділок (католицький)"),
            ("2022-04-22", "Страсна пʼятниця (православна)"),
            ("2022-04-24", "Великдень (православний)"),
            ("2022-04-25", "Великодній понеділок (православний)"),
            ("2022-05-01", "Міжнародний день праці"),
            ("2022-05-02", "Міжнародний день праці; Рамазан-байрам"),
            ("2022-05-03", "Міжнародний день праці (вихідний); Рамазан-байрам"),
            ("2022-05-09", "День перемоги над фашизмом"),
            ("2022-07-09", "Курбан-байрам"),
            ("2022-07-10", "Курбан-байрам"),
            (
                "2022-11-21",
                "День укладання Загальної рамкової угоди про мир у Боснії та Герцеговині",
            ),
            ("2022-11-25", "День державності"),
            ("2022-12-24", "Святий вечір (католицький)"),
            ("2022-12-25", "Різдво Христове (католицьке)"),
            ("2022-12-26", "Різдво Христове (католицьке) (вихідний)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.egypt import Egypt, EG, EGY
from tests.common import CommonCountryTests


class TestEgypt(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Egypt)

    def test_country_aliases(self):
        self.assertAliases(Egypt, EG, EGY)

    def test_2019(self):
        self.assertHoliday(
            "2019-01-07",
            "2019-01-25",
            "2019-04-25",
            "2019-04-28",
            "2019-04-29",
            "2019-05-01",
            "2019-07-23",
            "2019-10-6",
        )

    def test_siani_liberation_day(self):
        self.assertHoliday("1983-04-25")
        self.assertNoHoliday("1982-04-25")

    def test_revolution_day(self):
        self.assertHoliday("1953-07-23")
        self.assertNoHoliday("1952-07-23")

    def test_25_jan_from_2009(self):
        # Before 2009 Jan 25th wasn't celebrated
        self.assertHoliday("2010-01-25")
        self.assertNoHoliday("2008-01-25")

        self.assertHolidayName("عيد الشرطة", "2011-01-25")
        self.assertHolidayName("عيد ثورة 25 يناير", "2012-01-25")

    def test_hijri_based(self):
        self.assertHoliday(
            "2019-06-05",
            "2019-08-10",
            "2019-08-11",
            "2019-08-12",
            "2019-08-31",
            "2019-11-09",
            # Arafat
            "2010-02-26",
            # Eid al-Fitr
            "2010-09-10",
            "2019-06-04",
            # Eid al-Adha
            "2007-01-02",
            "2019-08-11",
            # Islamic New Year
            "2008-01-10",
            "2008-12-29",
            "2019-08-31",
            # Prophet Muhammad's Birthday
            "2010-02-26",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2019-01-01", "رأس السنة الميلادية"),
            ("2019-01-07", "عيد الميلاد المجيد (تقويم قبطي)"),
            ("2019-01-25", "عيد ثورة 25 يناير"),
            ("2019-04-25", "عيد تحرير سيناء"),
            ("2019-04-28", "عيد الفصح القبطي"),
            ("2019-04-29", "شم النسيم"),
            ("2019-05-01", "عيد العمال"),
            ("2019-06-04", "(تقدير) عيد الفطر"),
            ("2019-06-05", "(تقدير) عطلة عيد الفطر"),
            ("2019-06-06", "(تقدير) عطلة عيد الفطر"),
            ("2019-06-30", "عيد ثورة 30 يونيو"),
            ("2019-07-23", "عيد ثورة 23 يوليو"),
            ("2019-08-10", "(تقدير) يوم عرفة"),
            ("2019-08-11", "(تقدير) عيد الأضحى"),
            ("2019-08-12", "(تقدير) عطلة عيد الأضحى"),
            ("2019-08-13", "(تقدير) عطلة عيد الأضحى"),
            ("2019-08-31", "(تقدير) رأس السنة الهجرية"),
            ("2019-10-06", "عيد القوات المسلحة"),
            ("2019-11-09", "(تقدير) عيد المولد النبوي"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2019-01-01", "New Year's Day"),
            ("2019-01-07", "Coptic Christmas Day"),
            ("2019-01-25", "January 25th Revolution Day"),
            ("2019-04-25", "Sinai Liberation Day"),
            ("2019-04-28", "Coptic Easter"),
            ("2019-04-29", "Sham El Nessim"),
            ("2019-05-01", "Labor Day"),
            ("2019-06-04", "Eid al-Fitr (estimated)"),
            ("2019-06-05", "Eid al-Fitr Holiday (estimated)"),
            ("2019-06-06", "Eid al-Fitr Holiday (estimated)"),
            ("2019-06-30", "June 30 Revolution Day"),
            ("2019-07-23", "July 23 Revolution Day"),
            ("2019-08-10", "Arafat Day (estimated)"),
            ("2019-08-11", "Eid al-Adha (estimated)"),
            ("2019-08-12", "Eid al-Adha Holiday (estimated)"),
            ("2019-08-13", "Eid al-Adha Holiday (estimated)"),
            ("2019-08-31", "Islamic New Year (estimated)"),
            ("2019-10-06", "Armed Forces Day"),
            ("2019-11-09", "Prophet's Birthday (estimated)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.djibouti import Djibouti, DJ, DJI
from tests.common import CommonCountryTests


class TestDjibouti(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Djibouti)

    def test_country_aliases(self):
        self.assertAliases(Djibouti, DJ, DJI)

    def test_no_holidays(self):
        self.assertNoHolidays(Djibouti(years=1977))

    def test_2019(self):
        self.assertHolidays(
            ("2019-01-01", "Nouvel an"),
            ("2019-04-03", "Al Isra et Al Mirague (estimé)"),
            ("2019-05-01", "Fête du travail"),
            ("2019-06-04", "Eid al-Fitr (estimé)"),
            ("2019-06-05", "Eid al-Fitr deuxième jour (estimé)"),
            ("2019-06-27", "Fête de l'indépendance"),
            ("2019-06-28", "Fête de l'indépendance deuxième jour"),
            ("2019-08-10", "Arafat (estimé)"),
            ("2019-08-11", "Eid al-Adha (estimé)"),
            ("2019-08-12", "Eid al-Adha deuxième jour (estimé)"),
            ("2019-08-31", "Nouvel an musulman (estimé)"),
            ("2019-11-09", "Anniversaire du prophète Muhammad (estimé)"),
            ("2019-12-25", "Noël"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Nouvel an"),
            ("2022-02-28", "Al Isra et Al Mirague (estimé)"),
            ("2022-05-01", "Fête du travail"),
            ("2022-05-02", "Eid al-Fitr (estimé)"),
            ("2022-05-03", "Eid al-Fitr deuxième jour (estimé)"),
            ("2022-06-27", "Fête de l'indépendance"),
            ("2022-06-28", "Fête de l'indépendance deuxième jour"),
            ("2022-07-08", "Arafat (estimé)"),
            ("2022-07-09", "Eid al-Adha (estimé)"),
            ("2022-07-10", "Eid al-Adha deuxième jour (estimé)"),
            ("2022-07-30", "Nouvel an musulman (estimé)"),
            ("2022-10-08", "Anniversaire du prophète Muhammad (estimé)"),
            ("2022-12-25", "Noël"),
        )

    def test_l10n_ar(self):
        self.assertLocalizedHolidays(
            "ar",
            ("2022-01-01", "يوم السنة الجديدة"),
            ("2022-02-28", "(تقدير) الإسراء والمعراج"),
            ("2022-05-01", "عيد العمال"),
            ("2022-05-02", "(تقدير) عيد الفطر"),
            ("2022-05-03", "(تقدير) عطلة عيد الفطر"),
            ("2022-06-27", "عيد الإستقلال"),
            ("2022-06-28", "عطلة عيد الاستقلال"),
            ("2022-07-08", "(تقدير) يوم عرفة"),
            ("2022-07-09", "(تقدير) عيد الأضحى"),
            ("2022-07-10", "(تقدير) عطلة عيد الأضحى"),
            ("2022-07-30", "(تقدير) رأس السنة الهجرية"),
            ("2022-10-08", "(تقدير) عيد المولد النبوي"),
            ("2022-12-25", "عيد الميلاد المجيد"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-02-28", "Isra' and Mi'raj (estimated)"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Eid al-Fitr (estimated)"),
            ("2022-05-03", "Eid al-Fitr Holiday (estimated)"),
            ("2022-06-27", "Independence Day"),
            ("2022-06-28", "Independence Day Holiday"),
            ("2022-07-08", "Arafat Day (estimated)"),
            ("2022-07-09", "Eid al-Adha (estimated)"),
            ("2022-07-10", "Eid al-Adha Holiday (estimated)"),
            ("2022-07-30", "Islamic New Year (estimated)"),
            ("2022-10-08", "Prophet Muhammad's Birthday (estimated)"),
            ("2022-12-25", "Christmas Day"),
        )



#!/usr/bin/env python3

#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import importlib
import inspect
import sys
from pathlib import Path

from lingva.extract import main as create_pot_file
from polib import pofile


class POGenerator:
    """Generates .po files for supported country/market entities."""

    @staticmethod
    def update_po_file(po_path: str, pot_path: str, package_version: str) -> None:
        """Merge .po file with .pot"""
        po_file = pofile(po_path)
        po_file_initial = po_file.copy()
        pot_file = pofile(pot_path)

        po_file.merge(pot_file)

        # Update the project version if po file entries translation has changed only.
        if po_file != po_file_initial:
            po_file.metadata["Project-Id-Version"] = f"Holidays {package_version}"

        # Save the file each time in order to capture all other changes properly.
        po_file.save(po_path, newline="\n")

    def process_entities(self):
        """Processes entities in specified directory."""
        sys.path.append(f"{Path.cwd()}")  # Make holidays visible.
        from holidays import __version__ as package_version
        from holidays.holiday_base import HolidayBase

        entity_code_info_mapping = {}
        for entity_type in ("countries", "financial"):
            for path in Path(f"holidays/{entity_type}").glob("*.py"):
                if path.stem == "__init__":
                    continue
                module = f"holidays.{entity_type}.{path.stem}"
                for _, cls in inspect.getmembers(importlib.import_module(module), inspect.isclass):
                    if (
                        issubclass(cls, HolidayBase)
                        and cls.__module__ == module
                        and getattr(cls, "default_language") is not None
                    ):
                        name = getattr(cls, "country", getattr(cls, "market", None))
                        entity_code_info_mapping[name.upper()] = (cls.default_language, path)
                        break

        locale_path = Path("holidays/locale")
        pot_path = locale_path / "pot"
        pot_path.mkdir(exist_ok=True)
        for entity_code in sorted(entity_code_info_mapping.keys()):
            default_language, class_file_path = entity_code_info_mapping[entity_code]
            pot_file_path = pot_path / f"{entity_code}.pot"
            # Create .pot file.
            create_pot_file(
                (
                    f"{class_file_path}",
                    "-k",
                    "tr",
                    "-o",
                    f"{pot_file_path}",
                    "--package-name",
                    "Holidays",
                    "--package-version",
                    package_version,
                    "--width",
                    "100",
                    "--no-location",
                ),
                standalone_mode=False,
            )

            # Update .pot file metadata.
            pot_file = pofile(pot_file_path)
            pot_file.metadata.update(
                {
                    "Language": default_language,
                    "Language-Team": "Holidays Localization Team",
                    "PO-Revision-Date": pot_file.metadata["POT-Creation-Date"],
                    "X-Source-Language": default_language,
                }
            )
            pot_file.save()

            # Create entity default .po file from the .pot file.
            po_directory = locale_path / default_language / "LC_MESSAGES"
            po_directory.mkdir(parents=True, exist_ok=True)
            po_file_path = po_directory / f"{entity_code}.po"
            if not po_file_path.exists():
                pofile(pot_file_path).save(po_file_path)

            # Update all .po files.
            for po_file_path in locale_path.rglob(f"{entity_code}.po"):
                self.update_po_file(po_file_path, pot_file_path, package_version)

    @staticmethod
    def run():
        """Runs the .po files generation process."""
        POGenerator().process_entities()


if __name__ == "__main__":
    POGenerator.run()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import PUBLIC, SCHOOL
from holidays.countries.bulgaria import Bulgaria, BG, BLG
from tests.common import CommonCountryTests


class TestBulgaria(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Bulgaria, years=range(1990, 2050), years_non_observed=range(2017, 2030))

    def test_country_aliases(self):
        self.assertAliases(Bulgaria, BG, BLG)

    def test_no_holidays(self):
        self.assertNoHolidays(Bulgaria(categories=(PUBLIC, SCHOOL), years=1989))

    def test_new_years_day(self):
        name = "Нова година"
        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1990, 2050)))
        dt = (
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
            "2028-01-03",
        )
        self.assertHolidayName(f"{name} (почивен ден)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_liberation_day(self):
        name = "Ден на Освобождението на България от османско иго"
        self.assertHolidayName(name, (f"{year}-03-03" for year in range(1990, 2050)))
        dt = (
            "2018-03-05",
            "2019-03-04",
            "2024-03-04",
            "2029-03-05",
        )
        self.assertHolidayName(f"{name} (почивен ден)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_good_friday(self):
        self.assertHolidayName(
            "Велики петък",
            "2019-04-26",
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
        )

    def test_easter_saturday(self):
        self.assertHolidayName(
            "Велика събота",
            "2019-04-27",
            "2020-04-18",
            "2021-05-01",
            "2022-04-23",
            "2023-04-15",
        )

    def test_easter(self):
        self.assertHolidayName(
            "Великден",
            "2019-04-28",
            "2019-04-29",
            "2020-04-19",
            "2020-04-20",
            "2021-05-02",
            "2021-05-03",
            "2022-04-24",
            "2022-04-25",
            "2023-04-16",
            "2023-04-17",
        )

    def test_labour_day(self):
        name = "Ден на труда и на международната работническа солидарност"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1990, 2050)))
        dt = (
            "2021-05-04",
            "2022-05-02",
            "2027-05-04",
        )
        self.assertHolidayName(f"{name} (почивен ден)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_saint_georges_day(self):
        name = "Гергьовден, Ден на храбростта и Българската армия"
        self.assertHolidayName(name, (f"{year}-05-06" for year in range(1990, 2050)))
        dt = (
            "2017-05-08",
            "2018-05-07",
            "2023-05-08",
            "2028-05-08",
            "2029-05-07",
        )
        self.assertHolidayName(f"{name} (почивен ден)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_twenty_fourth_of_may(self):
        name = (
            "Ден на светите братя Кирил и Методий, на българската азбука, "
            "просвета и култура и на славянската книжовност"
        )
        self.assertHolidayName(name, (f"{year}-05-24" for year in range(1990, 2050)))
        dt = (
            "2020-05-25",
            "2025-05-26",
            "2026-05-25",
        )
        self.assertHolidayName(f"{name} (почивен ден)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_unification_day(self):
        name = "Ден на Съединението"
        self.assertHolidayName(name, (f"{year}-09-06" for year in range(1990, 2050)))
        dt = (
            "2020-09-07",
            "2025-09-08",
            "2026-09-07",
        )
        self.assertHolidayName(f"{name} (почивен ден)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_independence_day(self):
        name = "Ден на Независимостта на България"
        self.assertHolidayName(name, (f"{year}-09-22" for year in range(1990, 2050)))
        dt = (
            "2018-09-24",
            "2019-09-23",
            "2024-09-23",
            "2029-09-24",
        )
        self.assertHolidayName(f"{name} (почивен ден)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_national_awakening_day(self):
        name = "Ден на народните будители"
        self.assertHolidayName(
            name,
            Bulgaria(categories=SCHOOL, years=range(1990, 2050)),
            (f"{year}-11-01" for year in range(1990, 2050)),
        )
        self.assertNoHolidayName(name)

    def test_christmas_eve(self):
        name = "Бъдни вечер"
        self.assertHolidayName(name, (f"{year}-12-24" for year in range(1990, 2050)))
        dt = (
            "2017-12-27",
            "2022-12-27",
            "2023-12-27",
            "2028-12-27",
        )
        self.assertHolidayName(f"{name} (почивен ден)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_christmas_day(self):
        name = "Рождество Христово"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(1990, 2050)))
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(1990, 2050)))
        dt = (
            "2020-12-28",
            "2021-12-27",
            "2021-12-28",
            "2022-12-28",
            "2026-12-28",
            "2027-12-27",
            "2027-12-28",
        )
        self.assertHolidayName(f"{name} (почивен ден)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Нова година"),
            ("2022-01-03", "Нова година (почивен ден)"),
            ("2022-03-03", "Ден на Освобождението на България от османско иго"),
            ("2022-04-22", "Велики петък"),
            ("2022-04-23", "Велика събота"),
            ("2022-04-24", "Великден"),
            ("2022-04-25", "Великден"),
            ("2022-05-01", "Ден на труда и на международната работническа солидарност"),
            (
                "2022-05-02",
                "Ден на труда и на международната работническа солидарност (почивен ден)",
            ),
            ("2022-05-06", "Гергьовден, Ден на храбростта и Българската армия"),
            (
                "2022-05-24",
                "Ден на светите братя Кирил и Методий, на българската азбука, "
                "просвета и култура и на славянската книжовност",
            ),
            ("2022-09-06", "Ден на Съединението"),
            ("2022-09-22", "Ден на Независимостта на България"),
            ("2022-11-01", "Ден на народните будители"),
            ("2022-12-24", "Бъдни вечер"),
            ("2022-12-25", "Рождество Христово"),
            ("2022-12-26", "Рождество Христово"),
            ("2022-12-27", "Бъдни вечер (почивен ден)"),
            ("2022-12-28", "Рождество Христово (почивен ден)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-03-03", "Liberation Day"),
            ("2022-04-22", "Good Friday"),
            ("2022-04-23", "Holy Saturday"),
            ("2022-04-24", "Easter"),
            ("2022-04-25", "Easter"),
            ("2022-05-01", "Labor Day and International Workers' Solidarity Day"),
            ("2022-05-02", "Labor Day and International Workers' Solidarity Day (observed)"),
            ("2022-05-06", "Saint George's Day (Day of the Bulgarian Army)"),
            ("2022-05-24", "Day of Slavonic Alphabet, Bulgarian Enlightenment and Culture"),
            ("2022-09-06", "Unification Day"),
            ("2022-09-22", "Independence Day"),
            ("2022-11-01", "The Day of the People's Awakeners"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day"),
            ("2022-12-27", "Christmas Eve (observed)"),
            ("2022-12-28", "Christmas Day (observed)"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-03", "Новий рік (вихідний)"),
            ("2022-03-03", "День визволення Болгарії від османського іга"),
            ("2022-04-22", "Страсна пʼятниця"),
            ("2022-04-23", "Велика субота"),
            ("2022-04-24", "Великдень"),
            ("2022-04-25", "Великдень"),
            ("2022-05-01", "День праці та міжнародної солідарності трудящих"),
            ("2022-05-02", "День праці та міжнародної солідарності трудящих (вихідний)"),
            ("2022-05-06", "День Святого Георгія та День хоробрості і болгарської армії"),
            (
                "2022-05-24",
                "День святих братів Кирила і Мефодія, болгарської писемності, "
                "освіти і культури та словʼянської літератури",
            ),
            ("2022-09-06", "День обʼєднання"),
            ("2022-09-22", "День незалежності Болгарїі"),
            ("2022-11-01", "День національних будителів"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Різдво Христове"),
            ("2022-12-27", "Святий вечір (вихідний)"),
            ("2022-12-28", "Різдво Христове (вихідний)"),
        )



#!/usr/bin/env python3

#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import argparse
import json
import shutil
import sys
import warnings
from pathlib import Path

sys.path.append(f"{Path.cwd()}")  # Make holidays visible.

import holidays  # noqa: E402
from holidays import list_supported_countries, list_supported_financial  # noqa: E402


class SnapshotGenerator:
    """Creates a snapshot of available holidays for supported entities."""

    years = range(1950, 2051)

    def __init__(self) -> None:
        arg_parser = argparse.ArgumentParser()
        arg_parser.add_argument(
            "-c",
            "--country",
            action="extend",
            nargs="+",
            default=[],
            help="Country codes to use for snapshot generation",
            required=False,
            type=str,
        )
        arg_parser.add_argument(
            "-m",
            "--market",
            action="extend",
            nargs="+",
            default=[],
            help="Market codes to use for snapshot generation",
            required=False,
            type=str,
        )
        self.args = arg_parser.parse_args()

    @staticmethod
    def prepare_snapshot_directory(snapshot_path):
        """Prepare a directory for snapshots."""
        path = Path(snapshot_path)
        shutil.rmtree(path, ignore_errors=True)
        path.mkdir(parents=True, exist_ok=True)

    @staticmethod
    def save(snapshot, file_path):
        with open(file_path, "w", newline="\n") as output:
            output.write(
                json.dumps({str(dt): name for dt, name in sorted(snapshot.items())}, indent=4)
            )
            output.write("\n")  # Get along with pre-commit.

    def generate_country_snapshots(self):
        """Generates country snapshots."""
        if self.args.market:
            return None

        supported_countries = list_supported_countries(include_aliases=False)
        country_list = self.args.country or supported_countries
        if unknown_countries := set(country_list).difference(supported_countries.keys()):
            raise ValueError(f"Countries {', '.join(unknown_countries)} not available")

        snapshot_path = "snapshots/countries"
        if not self.args.country:
            self.prepare_snapshot_directory(snapshot_path)
        for country_code in country_list:
            country = getattr(holidays, country_code)

            for subdiv in (None,) + country.subdivisions:
                self.save(
                    holidays.country_holidays(
                        country_code,
                        subdiv=subdiv,
                        years=self.years,
                        categories=country.supported_categories,
                        language="en_US",
                    ),
                    f"{snapshot_path}/"
                    f"{country_code}_{(subdiv or 'COMMON').replace(' ', '_').upper()}.json",
                )

    def generate_financial_snapshots(self):
        """Generates financial snapshots."""
        if self.args.country:
            return None

        supported_markets = list_supported_financial(include_aliases=False)
        market_list = self.args.market or supported_markets
        if unknown_markets := set(market_list).difference(supported_markets.keys()):
            raise ValueError(f"Markets {', '.join(unknown_markets)} not available")

        snapshot_path = "snapshots/financial"
        if not self.args.market:
            self.prepare_snapshot_directory(snapshot_path)
        for market_code in market_list:
            self.save(
                holidays.country_holidays(
                    market_code,
                    years=self.years,
                    language="en_US",
                ),
                f"{snapshot_path}/{market_code}.json",
            )

    def run(self):
        """Runs snapshot files generation process."""
        self.generate_country_snapshots()
        self.generate_financial_snapshots()


if __name__ == "__main__":
    warnings.simplefilter("ignore")
    SnapshotGenerator().run()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.chad import Chad, TD, TCD
from tests.common import CommonCountryTests


class TestChad(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Chad)

    def test_country_aliases(self):
        self.assertAliases(Chad, TD, TCD)

    def test_no_holidays(self):
        self.assertNoHolidays(Chad(years=1960))

    def test_special_holidays(self):
        self.assertHoliday("2021-04-23")

    def test_freedom_and_democracy_day(self):
        name = "Freedom and Democracy Day"
        self.assertHolidayName(name, (f"{year}-12-01" for year in range(1991, 2050)))
        self.assertNoHolidayName(name, Chad(years=range(1961, 1991)))
        self.assertNoHoliday(f"{year}-12-01" for year in set(range(1961, 1991)).difference({1976}))

    def test_observed(self):
        dt = (
            # New Year's Day
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
            # International Women's Day
            "2015-03-09",
            "2020-03-09",
            # Labour Day
            "2011-05-02",
            "2016-05-02",
            # Independence Day
            "2013-08-12",
            "2019-08-12",
            # Republic Day
            "2010-11-29",
            "2021-11-29",
            # Freedom and Democracy Day
            "2013-12-02",
            "2019-12-02",
        )
        self.assertHoliday(dt)
        self.assertNoNonObservedHoliday(dt)

    def test_2022(self):
        self.assertHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-03-08", "International Women's Day"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labour Day"),
            ("2022-05-02", "Eid al-Fitr; Labour Day (observed)"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-08-11", "Independence Day"),
            ("2022-10-08", "Mawlid"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-28", "Republic Day"),
            ("2022-12-01", "Freedom and Democracy Day"),
            ("2022-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.lesotho import Lesotho, LS, LSO
from tests.common import CommonCountryTests


class TestLesotho(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Lesotho, years=range(1996, 2040))

    def test_country_aliases(self):
        self.assertAliases(Lesotho, LS, LSO)

    def test_no_holidays(self):
        self.assertNoHolidays(Lesotho(years=1995))

    def test_special_holidays(self):
        self.assertHoliday("2002-05-25")

    def test_heroes_day(self):
        name = "Heroes Day"
        self.assertHolidayName(name, (f"{year}-04-04" for year in range(1996, 2003)))
        self.assertNoHolidayName(name, range(2003, 2040))
        self.assertNoHoliday(f"{year}-04-04" for year in range(2003, 2040))

    def test_africa_heroes_day(self):
        name = "Africa/Heroes Day"
        self.assertHolidayName(name, (f"{year}-05-25" for year in range(2003, 2040)))
        self.assertNoHolidayName(name, range(1996, 2003))
        self.assertNoHoliday(f"{year}-05-25" for year in range(1996, 2002))

    def test_kings_birthday(self):
        name = "King's Birthday"
        self.assertHolidayName(name, (f"{year}-05-02" for year in range(1996, 1998)))
        self.assertHolidayName(name, (f"{year}-07-17" for year in range(1998, 2040)))
        self.assertNoHoliday(f"{year}-05-02" for year in range(1998, 2040))
        self.assertNoHoliday(f"{year}-07-17" for year in range(1996, 1998))

    def test_2022(self):
        self.assertHolidays(
            Lesotho(years=2022),
            ("2022-01-01", "New Year's Day"),
            ("2022-03-11", "Moshoeshoe's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Workers' Day"),
            ("2022-05-25", "Africa/Heroes Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-07-17", "King's Birthday"),
            ("2022-10-04", "Independence Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.zimbabwe import Zimbabwe, ZW, ZWE
from tests.common import CommonCountryTests


class TestZimbabwe(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Zimbabwe, years=range(1988, 2050))

    def test_country_aliases(self):
        self.assertAliases(Zimbabwe, ZW, ZWE)

    def test_no_holidays(self):
        self.assertNoHolidays(Zimbabwe(years=1987))

    def test_holidays(self):
        for year in range(1988, 2050):
            self.assertHoliday(
                f"{year}-01-01",
                f"{year}-04-18",
                f"{year}-05-01",
                f"{year}-05-25",
                f"{year}-12-22",
                f"{year}-12-25",
                f"{year}-12-26",
            )

        self.assertNoHoliday(f"{year}-02-21" for year in range(1988, 2018))
        self.assertNoHolidayName("Robert Gabriel Mugabe National Youth Day", range(1988, 2018))
        self.assertHoliday(f"{year}-02-21" for year in range(2018, 2050))

    def test_easter(self):
        self.assertHoliday(
            # Good Friday
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            # Holy Saturday
            "2018-03-31",
            "2019-04-20",
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            # Easter Monday
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
        )

    def test_moving_holidays(self):
        self.assertHoliday(
            # Zimbabwe Heroes' Day
            "2018-08-13",
            "2019-08-12",
            "2020-08-10",
            "2021-08-09",
            "2022-08-08",
            # Defense Forces Day
            "2018-08-14",
            "2019-08-13",
            "2020-08-11",
            "2021-08-10",
            "2022-08-09",
        )

    def test_observed(self):
        dt = (
            "2002-12-23",
            "2003-05-26",
            "2004-04-19",
            "2004-12-27",
            "2005-05-02",
            "2005-12-27",
            "2006-01-02",
            "2008-05-26",
            "2010-04-19",
            "2010-12-27",
            "2011-05-02",
            "2011-12-27",
            "2012-01-02",
            "2013-12-23",
            "2014-05-26",
            "2016-05-02",
            "2016-12-27",
            "2017-01-02",
            "2019-12-23",
            "2021-02-22",
            "2021-04-19",
            "2021-12-27",
            "2022-05-02",
            "2022-12-27",
            "2023-01-02",
            "2024-12-23",
            "2027-02-22",
            # special cases
            "2049-04-20",
            "2055-04-20",
            "2060-04-20",
        )
        self.assertHoliday(dt)
        self.assertNoNonObservedHoliday(dt)

    def test_2022(self):
        self.assertHolidays(
            Zimbabwe(years=2022),
            ("2022-01-01", "New Year's Day"),
            ("2022-02-21", "Robert Gabriel Mugabe National Youth Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-16", "Easter Saturday"),
            ("2022-04-18", "Easter Monday; Independence Day"),
            ("2022-05-01", "Workers' Day"),
            ("2022-05-02", "Workers' Day (observed)"),
            ("2022-05-25", "Africa Day"),
            ("2022-08-08", "Zimbabwe Heroes' Day"),
            ("2022-08-09", "Defense Forces Day"),
            ("2022-12-22", "Unity Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import GOVERNMENT, PUBLIC
from holidays.countries.paraguay import Paraguay, PY, PRY
from tests.common import CommonCountryTests


class TestParaguay(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Paraguay, years=range(1991, 2050))

    def test_country_aliases(self):
        self.assertAliases(Paraguay, PY, PRY)

    def test_no_holidays(self):
        self.assertNoHolidays(Paraguay(categories=(GOVERNMENT, PUBLIC), years=1990))

    def test_new_years_day(self):
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in range(1991, 2050)))

    def test_patriots_day(self):
        name = "Día de los Héroes de la Patria"
        years_excluded = {2013, 2016, 2018, 2019, 2022, 2023}
        self.assertHolidayName(
            name, (f"{year}-03-01" for year in set(range(1991, 2050)).difference(years_excluded))
        )
        self.assertNoHoliday(f"{year}-03-01" for year in years_excluded)
        self.assertHolidayName(
            name,
            "2013-03-04",
            "2016-02-29",
            "2018-02-26",
            "2019-03-04",
            "2022-02-28",
            "2023-02-27",
        )

    def test_easter(self):
        self.assertHolidayName(
            "Jueves Santo",
            "2019-04-18",
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
        )

        self.assertHolidayName(
            "Viernes Santo",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )

        self.assertHolidayName(
            "Domingo de Resurrección",
            "2019-04-21",
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
        )

    def test_labor_day(self):
        self.assertHolidayName(
            "Día de los Trabajadores", (f"{year}-05-01" for year in range(1991, 2050))
        )

    def test_national_holiday(self):
        name = "Feriado Nacional"
        self.assertHolidayName(name, (f"{year}-05-14" for year in range(2012, 2050)))
        # 2011 - special holiday
        self.assertNoHoliday(f"{year}-05-14" for year in range(1991, 2011))
        self.assertNoHolidayName(name, range(1991, 2012))

    def test_independence_day(self):
        self.assertHolidayName(
            "Día de la Independencia Nacional", (f"{year}-05-15" for year in range(1991, 2050))
        )

    def test_chaco_armistice_day(self):
        name = "Día de la Paz del Chaco"
        years_excluded = {2014, 2018, 2019, 2024}
        self.assertHolidayName(
            name, (f"{year}-06-12" for year in set(range(1991, 2050)).difference(years_excluded))
        )
        self.assertNoHoliday(f"{year}-06-12" for year in years_excluded)
        self.assertHolidayName(
            name,
            "2014-06-16",
            "2018-06-11",
            "2019-06-17",
            "2024-06-10",
        )

    def test_asuncion_foundations_day(self):
        self.assertHolidayName(
            "Día de la Fundación de Asunción", (f"{year}-08-15" for year in range(1991, 2050))
        )

    def test_boqueron_battle_day(self):
        name = "Día de la Batalla de Boquerón"
        years_excluded = {2015, 2016, 2017, 2021, 2022, 2024}
        self.assertHolidayName(
            name, (f"{year}-09-29" for year in set(range(1995, 2050)).difference(years_excluded))
        )
        self.assertNoHoliday(f"{year}-09-29" for year in years_excluded)
        self.assertNoHoliday(f"{year}-09-29" for year in range(1991, 1995))
        self.assertNoHolidayName(name, range(1991, 1995))
        self.assertHolidayName(
            name,
            "2015-09-28",
            "2016-10-03",
            "2017-10-02",
            "2021-09-27",
            "2022-10-03",
            "2024-09-30",
        )

    def test_caacupe_virgin_day(self):
        self.assertHolidayName(
            "Día de la Virgen de Caacupé", (f"{year}-12-08" for year in range(1991, 2050))
        )

    def test_special_public_holidays(self):
        self.assertHoliday(
            "2007-01-29",
            "2009-09-10",
            "2010-06-14",
            "2011-04-19",
            "2011-05-14",
            "2011-05-16",
            "2013-08-14",
            "2015-07-10",
        )

    def test_special_government_holidays(self):
        self.assertHoliday(
            Paraguay(categories=GOVERNMENT, years=range(2010, 2023)),
            "2010-12-24",
            "2010-12-31",
            "2011-04-20",
            "2011-12-23",
            "2011-12-30",
            "2012-04-04",
            "2012-12-24",
            "2012-12-31",
            "2013-03-27",
            "2014-04-16",
            "2014-12-24",
            "2014-12-31",
            "2015-04-01",
            "2015-12-24",
            "2015-12-31",
            "2016-03-23",
            "2017-03-28",
            "2018-12-24",
            "2018-12-31",
            "2019-04-17",
            "2019-12-24",
            "2019-12-31",
            "2020-04-08",
            "2021-12-24",
            "2021-12-31",
            "2022-04-13",
            "2022-05-02",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-02-28", "Día de los Héroes de la Patria"),
            ("2022-04-13", "Asueto de la Administración Pública"),
            ("2022-04-14", "Jueves Santo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-04-17", "Domingo de Resurrección"),
            ("2022-05-01", "Día de los Trabajadores"),
            ("2022-05-02", "Asueto de la Administración Pública"),
            ("2022-05-14", "Feriado Nacional"),
            ("2022-05-15", "Día de la Independencia Nacional"),
            ("2022-06-12", "Día de la Paz del Chaco"),
            ("2022-08-15", "Día de la Fundación de Asunción"),
            ("2022-10-03", "Día de la Batalla de Boquerón"),
            ("2022-12-08", "Día de la Virgen de Caacupé"),
            ("2022-12-25", "Día de la Navidad"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-02-28", "Patriots Day"),
            ("2022-04-13", "Public sector holiday"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-05-01", "Workers' Day"),
            ("2022-05-02", "Public sector holiday"),
            ("2022-05-14", "National Holiday"),
            ("2022-05-15", "Independence Day"),
            ("2022-06-12", "Chaco Armistice Day"),
            ("2022-08-15", "Asuncion Foundation's Day"),
            ("2022-10-03", "Boqueron Battle Day"),
            ("2022-12-08", "Caacupe Virgin Day"),
            ("2022-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-02-28", "День національних героїв"),
            ("2022-04-13", "Вихідний державних установ"),
            ("2022-04-14", "Великий четвер"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-05-01", "День трудящих"),
            ("2022-05-02", "Вихідний державних установ"),
            ("2022-05-14", "Національне свято"),
            ("2022-05-15", "День незалежності"),
            ("2022-06-12", "День мирного договору в Чако"),
            ("2022-08-15", "День заснування Асунсьйона"),
            ("2022-10-03", "День битви за Бокерон"),
            ("2022-12-08", "День Богоматері Каакупе"),
            ("2022-12-25", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.burkina_faso import BurkinaFaso, BF, BFA
from tests.common import CommonCountryTests


class TestBurkinaFaso(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(BurkinaFaso)

    def test_country_aliases(self):
        self.assertAliases(BurkinaFaso, BF, BFA)

    def test_no_holidays(self):
        self.assertNoHolidays(BurkinaFaso(years=1960))

    def test_revolution_day(self):
        name = "Revolution Day"
        self.assertHolidayName(name, (f"{year}-01-03" for year in range(1967, 2050)))
        self.assertNoHolidayName(name, BurkinaFaso(years=range(1961, 1967)))
        self.assertNoHoliday(f"{year}-01-03" for year in range(1961, 1967))

    def test_martyrs_day(self):
        name = "Martyrs' Day"
        self.assertHolidayName(name, (f"{year}-10-31" for year in range(2016, 2050)))
        self.assertNoHolidayName(name, BurkinaFaso(years=range(1961, 2016)))
        self.assertNoHoliday(f"{year}-10-31" for year in set(range(1961, 2016)).difference({1979}))

    def test_observed(self):
        dt = (
            # New Year's Day
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
            # Revolution Day
            "2010-01-04",
            "2016-01-04",
            "2021-01-04",
            # International Women's Day
            "2015-03-09",
            "2020-03-09",
            # Labour Day
            "2011-05-02",
            "2016-05-02",
            # Independence Day
            "2012-08-06",
            "2018-08-06",
            # Assumption Day
            "2010-08-16",
            "2021-08-16",
            # All Saints' Day
            "2015-11-02",
            "2020-11-02",
            # Proclamation of Independence Day
            "2011-12-12",
            "2022-12-12",
            # Christmas Day
            "2011-12-26",
            "2016-12-26",
            "2022-12-26",
        )
        self.assertHoliday(dt)
        self.assertNoNonObservedHoliday(dt)

    def test_2022(self):
        self.assertHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "Revolution Day"),
            ("2022-03-08", "International Women's Day"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labour Day"),
            ("2022-05-02", "Eid al-Fitr; Labour Day (observed)"),
            ("2022-05-26", "Ascension Day"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-08-05", "Independence Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-10-09", "Mawlid"),
            ("2022-10-31", "Martyrs' Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-12-11", "Proclamation of Independence Day"),
            ("2022-12-12", "Proclamation of Independence Day (observed)"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import ARMED_FORCES, HALF_DAY, PUBLIC
from holidays.countries.palau import Palau, PW, PLW
from tests.common import CommonCountryTests


class TestPalau(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Palau, years=range(1981, 2040), years_non_observed=range(1981, 2040))

    def test_country_aliases(self):
        self.assertAliases(Palau, PW, PLW)

    def test_no_holidays(self):
        self.assertNoHolidays(Palau(years=1980))

    def test_special_holidays(self):
        self.assertHoliday(
            "2020-11-03",
        )

    def test_armed_forces_holiday(self):
        self.assertHolidays(
            Palau(categories=ARMED_FORCES, years=2020), ("2020-11-11", "Veterans Day")
        )

    def test_half_day_holiday(self):
        self.assertHolidays(
            Palau(categories=HALF_DAY, years=2019),
            ("2019-09-30", "Preparation for the 25th Independence Day of the Republic of Palau"),
        )

    def test_new_years_day_observed(self):
        self.assertNoNonObservedHoliday(
            "1982-12-31",
            "1993-12-31",
            "1999-12-31",
            "2004-12-31",
            "2010-12-31",
            "2021-12-31",
        )

    def test_memorial_day(self):
        name = "Memorial Day"
        dt = (
            # Last Monday of May.
            "2011-05-30",
            "2012-05-28",
        )
        self.assertHolidayName(name, dt)
        self.assertNoHolidayName(name, range(1981, 2011), range(2013, 2040))

    def test_family_day(self):
        name = "Family Day"
        dt = (
            # 4th Friday of Nov.
            "2017-11-24",
            "2018-11-23",
            "2019-11-22",
            "2020-11-27",
            "2021-11-26",
            "2022-11-25",
            "2023-11-24",
            "2024-11-22",
        )
        self.assertHolidayName(name, dt)
        self.assertNoHolidayName(name, range(1981, 2017))

    def test_presidents_day(self):
        name = "President's Day"
        self.assertHolidayName(name, (f"{year}-06-01" for year in range(2018, 2040)))
        self.assertNoHolidayName(name, range(1981, 2018))
        self.assertNoNonObservedHoliday(
            "2019-05-30",
            "2024-05-30",
            "2025-06-02",
            "2030-05-30",
            "2031-06-02",
        )

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-10-01" for year in range(2018, 2040)))
        self.assertNoHolidayName(name, range(1981, 2018))
        self.assertNoNonObservedHoliday(
            "2022-09-30",
            "2023-10-02",
            "2028-10-02",
            "2033-09-30",
            "2034-10-02",
        )

    def test_2020(self):
        # https://www.facebook.com/photo/?fbid=1499007180251376&set=a.175933635892077
        self.assertHolidays(
            Palau(categories=(ARMED_FORCES, PUBLIC), years=2020),
            ("2020-01-01", "New Year's Day"),
            ("2020-03-15", "Youth Day"),
            ("2020-03-16", "Youth Day (observed)"),
            ("2020-05-05", "Senior Citizens Day"),
            ("2020-06-01", "President's Day"),
            ("2020-07-09", "Constitution Day"),
            ("2020-09-07", "Labor Day"),
            ("2020-10-01", "Independence Day"),
            ("2020-10-23", "United Nations Day (observed)"),
            ("2020-10-24", "United Nations Day"),
            ("2020-11-03", "National Day of Democracy"),
            ("2020-11-11", "Veterans Day"),
            ("2020-11-26", "Thanksgiving Day"),
            ("2020-11-27", "Family Day"),
            ("2020-12-25", "Christmas Day"),
        )

    def test_2022(self):
        # https://www.facebook.com/photo/?fbid=2076492362502852&set=a.175933635892077
        self.assertHolidays(
            Palau(years=2022),
            ("2022-01-01", "New Year's Day"),
            ("2022-03-15", "Youth Day"),
            ("2022-05-05", "Senior Citizens Day"),
            ("2022-06-01", "President's Day"),
            ("2022-07-08", "Constitution Day (observed)"),
            ("2022-07-09", "Constitution Day"),
            ("2022-09-05", "Labor Day"),
            ("2022-09-30", "Independence Day (observed)"),
            ("2022-10-01", "Independence Day"),
            ("2022-10-24", "United Nations Day"),
            ("2022-11-24", "Thanksgiving Day"),
            ("2022-11-25", "Family Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
        )

    def test_2023(self):
        # https://www.facebook.com/photo/?fbid=510442897789296&set=a.368725625294358
        self.assertHolidays(
            Palau(years=2023),
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-03-15", "Youth Day"),
            ("2023-05-05", "Senior Citizens Day"),
            ("2023-06-01", "President's Day"),
            ("2023-07-09", "Constitution Day"),
            ("2023-07-10", "Constitution Day (observed)"),
            ("2023-09-04", "Labor Day"),
            ("2023-10-01", "Independence Day"),
            ("2023-10-02", "Independence Day (observed)"),
            ("2023-10-24", "United Nations Day"),
            ("2023-11-23", "Thanksgiving Day"),
            ("2023-11-24", "Family Day"),
            ("2023-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from unittest import TestCase

from holidays.countries.serbia import Serbia, RS, SRB
from tests.common import CommonCountryTests


class TestSerbia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Serbia)

    def test_country_aliases(self):
        self.assertAliases(Serbia, RS, SRB)

    def test_new_year(self):
        # If January 1st is in Weekend, test oberved
        self.assertIn(date(2017, 1, 1), self.holidays)
        self.assertIn(date(2017, 1, 2), self.holidays)
        self.assertIn(date(2017, 1, 3), self.holidays)
        self.holidays.observed = False
        self.assertNotIn(date(2017, 1, 3), self.holidays)

    def test_statehood_day(self):
        # If February 15th is in Weekend, test oberved
        self.assertIn(date(2020, 2, 15), self.holidays)
        self.assertIn(date(2020, 2, 16), self.holidays)
        self.assertIn(date(2020, 2, 17), self.holidays)
        self.holidays.observed = False
        self.assertNotIn(date(2020, 2, 17), self.holidays)

    def test_labour_day(self):
        # If May 1st is in Weekend, test oberved
        self.assertIn(date(2016, 5, 1), self.holidays)
        self.assertIn(date(2016, 5, 2), self.holidays)
        self.assertIn(date(2016, 5, 3), self.holidays)
        self.assertIn(date(2021, 5, 1), self.holidays)
        self.assertIn(date(2021, 5, 2), self.holidays)
        self.assertIn(date(2021, 5, 3), self.holidays)
        self.assertIn(date(2021, 5, 4), self.holidays)
        self.holidays.observed = False
        self.assertNotIn(date(2016, 5, 3), self.holidays)
        self.assertIn(date(2021, 5, 3), self.holidays)
        self.assertNotIn(date(2021, 5, 4), self.holidays)

    def test_armistice_day(self):
        # If November 11th is in Weekend, test oberved
        self.assertIn(date(2018, 11, 11), self.holidays)
        self.assertIn(date(2018, 11, 12), self.holidays)
        self.holidays.observed = False
        self.assertNotIn(date(2018, 11, 12), self.holidays)

    def test_religious_holidays(self):
        # Orthodox Christmas
        self.assertIn(date(2020, 1, 7), self.holidays)
        self.assertNotIn(date(2020, 1, 8), self.holidays)
        # Orthodox Easter
        self.assertNotIn(date(2020, 4, 16), self.holidays)
        self.assertIn(date(2020, 4, 17), self.holidays)
        self.assertIn(date(2020, 4, 18), self.holidays)
        self.assertIn(date(2020, 4, 19), self.holidays)
        self.assertIn(date(2020, 4, 20), self.holidays)
        self.assertNotIn(date(2020, 4, 21), self.holidays)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Нова година"),
            ("2022-01-02", "Нова година"),
            ("2022-01-03", "Нова година (слободан дан)"),
            ("2022-01-07", "Божић"),
            ("2022-02-15", "Дан државности Србије"),
            ("2022-02-16", "Дан државности Србије"),
            ("2022-04-22", "Велики петак"),
            ("2022-04-23", "Велика субота"),
            ("2022-04-24", "Васкрс"),
            ("2022-04-25", "Други дан Васкрса"),
            ("2022-05-01", "Празник рада"),
            ("2022-05-02", "Празник рада"),
            ("2022-05-03", "Празник рада (слободан дан)"),
            ("2022-11-11", "Дан примирја у Првом светском рату"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-02", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-01-07", "Orthodox Christmas Day"),
            ("2022-02-15", "Statehood Day"),
            ("2022-02-16", "Statehood Day"),
            ("2022-04-22", "Good Friday"),
            ("2022-04-23", "Holy Saturday"),
            ("2022-04-24", "Easter Sunday"),
            ("2022-04-25", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Labor Day"),
            ("2022-05-03", "Labor Day (observed)"),
            ("2022-11-11", "Armistice Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import ARMED_FORCES, BANK, GOVERNMENT, PUBLIC, SCHOOL, WORKDAY
from holidays.countries.thailand import Thailand, TH, THA
from tests.common import CommonCountryTests


class TestThailand(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        support_range = range(1914, 2050)
        super().setUpClass(Thailand, years=support_range, years_non_observed=support_range)

    def test_country_aliases(self):
        self.assertAliases(Thailand, TH, THA)

    def test_no_holidays(self):
        self.assertNoHolidays(Thailand(years=1913, categories=(PUBLIC, WORKDAY)))
        self.assertNoHolidays(Thailand(years=1942, categories=BANK))
        self.assertNoHolidays(Thailand(years=1956, categories=(GOVERNMENT, SCHOOL)))
        self.assertNoHolidays(Thailand(years=1958, categories=ARMED_FORCES))

    def test_special_holidays(self):
        dt = (
            # 1992-1994 (include In Lieus, Checked with Bank of Thailand Data).
            "1992-05-18",
            "1992-12-07",
            "1993-03-08",
            "1993-05-03",
            "1993-10-25",
            "1993-12-06",
            "1994-01-03",
            "1994-05-02",
            "1994-07-25",
            "1994-10-24",
            "1994-12-12",
            # 1995-1997 (Bank of Thailand Data).
            "1996-06-10",
            # 1998-2000 (include In Lieus, Checked with Bank of Thailand Data).
            "1998-05-11",
            "1998-12-07",
            "1999-05-03",
            "1999-05-31",
            "1999-10-25",
            "1999-12-06",
            "2000-01-03",
            "2000-02-21",
            "2000-08-14",
            "2000-12-11",
            "2000-12-29",
            # From 2001 Onwards (Checked with Bank of Thailand Data).
            "2006-04-19",
            "2006-06-09",
            "2006-06-12",
            "2006-06-13",
            "2006-09-20",
            "2009-01-02",
            "2009-04-10",
            "2009-04-16",
            "2009-04-17",
            "2009-07-06",
            "2010-05-20",
            "2010-05-21",
            "2010-08-13",
            "2011-05-16",
            "2011-10-27",
            "2011-10-28",
            "2011-10-29",
            "2011-10-30",
            "2011-10-31",
            "2012-04-09",
            "2013-12-30",
            "2014-08-11",
            "2015-01-02",
            "2015-05-04",
            "2016-05-06",
            "2016-07-18",
            "2016-10-14",
            "2017-10-26",
            "2019-05-06",
            "2020-11-19",
            "2020-11-20",
            "2020-12-11",
            "2021-02-12",
            "2021-04-12",
            "2021-09-24",
            "2022-07-15",
            "2022-07-29",
            "2022-10-14",
            "2022-12-30",
            "2023-05-05",
            "2023-07-31",
            "2023-12-29",
            "2024-04-12",
            "2024-12-30",
            "2025-06-02",
            "2025-08-11",
            "2026-01-02",
        )
        dt_observed = (
            "2007-12-24",
            "2020-07-27",
            "2020-09-04",
            "2020-09-07",
        )
        self.assertHoliday(dt, dt_observed)
        self.assertNoNonObservedHoliday(dt_observed)

    def test_new_years_day(self):
        name = "วันขึ้นปีใหม่"

        self.assertHolidayName(name, (f"{year}-01-01" for year in range(1941, 2050)))
        self.assertHolidayName(
            name,
            (f"{year}-01-02" for year in range(1941, 1945)),
            (f"{year}-01-02" for year in range(1948, 1954)),
        )
        self.assertNoHolidayName(name, range(1914, 1941))

        self.assertNoNonObservedHoliday(
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
            "2028-01-03",
        )

    def test_national_childrens_day(self):
        name = "วันเด็กแห่งชาติ"
        dt = (
            # First Iteration
            "1955-10-03",
            # Second and Current Iteration
            "2010-01-09",
            "2011-01-08",
            "2012-01-14",
            "2013-01-12",
            "2014-01-11",
            "2015-01-10",
            "2016-01-09",
            "2017-01-14",
            "2018-01-13",
            "2019-01-12",
            "2020-01-11",
            "2021-01-09",
            "2022-01-08",
            "2023-01-14",
            "2024-01-13",
            "2025-01-11",
        )
        self.assertHolidayName(name, dt)
        self.assertNoHolidayName(name, range(1914, 1955), 1964)

    def test_franco_thai_war_armistice_day(self):
        name = "วันลงนามในสัญญาพักรบระหว่างประเทศไทยกับประเทศอินโดจีนฝรั่งเศส"
        self.assertHolidayName(name, (f"{year}-01-28" for year in range(1942, 1945)))
        self.assertNoHolidayName(name, range(1914, 1942), range(1945, 2050))

    def test_chakri_memorial_day(self):
        name_maha_chakri = "วันที่ระลึกมหาจักรี"
        name_chakri = "วันจักรี"
        name_rama_one = "วันพระบาทสมเด็จพระพุทธยอดฟ้าจุฬาโลกมหาราช และวันที่ระลึกมหาจักรีบรมราชวงศ์"
        years_pre = range(1914, 1926)
        years_maha_chakri = range(1926, 1938)
        years_chakri = range(1938, 1983)
        years_rama_one = range(1983, 2050)

        self.assertHolidayName(name_maha_chakri, (f"{year}-04-06" for year in years_maha_chakri))
        self.assertHolidayName(name_chakri, (f"{year}-04-06" for year in years_chakri))
        self.assertHolidayName(name_rama_one, (f"{year}-04-06" for year in years_rama_one))
        self.assertNoHolidayName(name_maha_chakri, years_pre, years_chakri, years_rama_one)
        self.assertNoHolidayName(name_chakri, years_pre, years_maha_chakri, years_rama_one)
        self.assertNoHolidayName(name_rama_one, years_pre, years_maha_chakri, years_chakri)

        self.assertNoNonObservedHoliday(
            "2013-04-08",
            "2014-04-07",
            "2019-04-08",
            "2024-04-08",
            "2025-04-07",
            "2030-04-08",
        )

    def test_songkran_festival(self):
        name_1914 = "พระราชพิธีตะรุษะสงกรานต์ แลนักขัตฤกษ์"
        name_1926 = "ตะรุษะสงกรานต์"
        name_1938 = "วันตรุษสงกรานต์"
        name_1939 = "วันตรุษสงกรานต์และขึ้นปีใหม่"
        name_1948 = "วันสงกรานต์"

        # MAR 28 - APR 15
        for year in range(1914, 1926):
            self.assertHolidayName(name_1914, (f"{year}-03-28", f"{year}-04-01", f"{year}-04-15"))
        self.assertNoHolidayName(name_1914, range(1926, 2050))
        # MAR 31 - APR 3
        for year in range(1926, 1938):
            self.assertHolidayName(name_1926, (f"{year}-03-31", f"{year}-04-01", f"{year}-04-03"))
        self.assertNoHolidayName(name_1926, range(1914, 1926), range(1938, 2050))
        # MAR 28 - APR 2
        for year in range(1938, 1940):
            self.assertHolidayName(name_1938, (f"{year}-03-31", f"{year}-04-01", f"{year}-04-02"))
        self.assertHolidayName(name_1939, "1940-03-31", "1940-04-01", "1940-04-02")
        self.assertNoHolidayName(name_1938, range(1914, 1938), range(1940, 2050))
        self.assertNoHolidayName(name_1939, range(1914, 1940), range(1941, 2050))
        # APR 13-14-15
        for year in [*range(1948, 1954), *range(1998, 2020), *range(2021, 2050)]:
            self.assertHolidayName(name_1948, (f"{year}-04-13", f"{year}-04-14", f"{year}-04-15"))
        # APR 12-13-14
        for year in range(1989, 1998):
            self.assertHolidayName(name_1948, (f"{year}-04-12", f"{year}-04-13", f"{year}-04-14"))
        # APR 13
        self.assertHolidayName(name_1948, (f"{year}-04-13" for year in range(1957, 1989)))
        # None (2020 is special_public_holidays instead)
        self.assertNoHolidayName(name_1948, "2020-04-13", "2020-04-14", "2020-04-15")
        self.assertNoHolidayName(name_1948, range(1914, 1948), range(1954, 1957))

        self.assertNoNonObservedHoliday(
            "2012-04-16",
            "2013-04-16",
            "2014-04-16",
            "2017-04-17",
            "2018-04-16",
            "2019-04-16",
            # 2020 Songkran Festival special in lieus doesn't counts
            "2023-04-17",
            "2024-04-16",
            "2025-04-16",
            "2028-04-17",
            "2029-04-16",
            "2030-04-16",
        )

    def test_national_labour_day(self):
        name = "วันแรงงานแห่งชาติ"

        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1974, 2050)))
        self.assertNoHolidayName(name, range(1914, 1974))

        self.assertNoNonObservedHoliday(
            "2010-05-03",
            "2011-05-02",
            "2016-05-02",
            "2021-05-03",
            "2022-05-02",
            "2027-05-03",
        )

    def test_coronation_day(self):
        name_1914 = "ทำบุญพระบรมอัษฐิ และพระราชพิธีฉัตรมงคล"
        name_1926 = "พระราชพิธีฉัตรมงคล"
        name_1958 = "วันฉัตรมงคล"

        for year in range(1914, 1925):
            self.assertHolidayName(
                name_1914, (f"{year}-11-09", f"{year}-11-10", f"{year}-11-11", f"{year}-11-12")
            )
        self.assertNoHolidayName(name_1914, range(1926, 2050))
        for year in range(1926, 1936):
            self.assertHolidayName(name_1926, (f"{year}-02-24", f"{year}-02-25", f"{year}-02-26"))
        self.assertNoHolidayName(name_1926, range(1914, 1926), range(1936, 2050))
        self.assertHolidayName(name_1958, (f"{year}-05-05" for year in range(1958, 2017)))
        self.assertHolidayName(name_1958, (f"{year}-05-04" for year in range(2020, 2050)))
        self.assertNoHolidayName(name_1958, range(1914, 1958), range(2017, 2020))

        self.assertNoNonObservedHoliday(
            "2012-05-07",
            "2013-05-06",
            "2024-05-06",
            "2025-05-05",
            "2030-05-06",
        )

    def test_queen_suthida_birthday(self):
        name = "วันเฉลิมพระชนมพรรษาสมเด็จพระนางเจ้าสุทิดา พัชรสุธาพิมลลักษณ พระบรมราชินี"

        self.assertHolidayName(name, (f"{year}-06-03" for year in range(2019, 2050)))
        self.assertNoHolidayName(name, range(1914, 2019))

        self.assertNoNonObservedHoliday(
            "2023-06-05",
            "2028-06-05",
            "2029-06-04",
        )

    def test_national_day(self):
        name_1938 = "วันขอพระราชทานรัฐธรรมนูญ"
        name_1939 = "วันชาติ"

        self.assertHolidayName(name_1938, "1938-06-24")
        for year in [*range(1940, 1948), *range(1952, 1954)]:
            self.assertHolidayName(name_1939, (f"{year}-06-23", f"{year}-06-24", f"{year}-06-25"))
        self.assertHolidayName(
            name_1939,
            "1939-06-24",
            (f"{year}-06-24" for year in range(1948, 1952)),
            (f"{year}-06-24" for year in range(1954, 1960)),
        )
        self.assertHolidayName(name_1939, (f"{year}-12-05" for year in range(1960, 2050)))
        self.assertNoHolidayName(name_1938, range(1914, 1938), range(1939, 2050))
        self.assertNoHolidayName(name_1939, range(1914, 1939))

        # No in lieus during its existense on June 24th
        # 1960+ In lieus are same as HM King Bhumibol Adulyadej's Birthday

    def test_provisional_constitution_day(self):
        name = "วันรัฐธรรมนูญชั่วคราว"

        self.assertHolidayName(name, (f"{year}-06-27" for year in range(1938, 1940)))
        self.assertNoHolidayName(name, range(1914, 1938), range(1940, 2050))

    def test_rama_x_birthday(self):
        name = "วันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระปรเมนทรรามาธิบดีศรีสินทรมหาวชิราลงกรณ พระวชิรเกล้าเจ้าอยู่หัว"

        self.assertHolidayName(name, (f"{year}-07-28" for year in range(2017, 2050)))
        self.assertNoHolidayName(name, range(1914, 2017))

        self.assertNoNonObservedHoliday(
            "2018-07-30",
            "2019-07-29",
            "2024-07-29",
            "2029-07-30",
            "2030-07-29",
        )

    def test_queen_sirikit_birthday(self):
        name_ix = "วันเฉลิมพระชนมพรรษาสมเด็จพระนางเจ้าสิริกิติ์ พระบรมราชินีนาถ"
        name_x = "วันเฉลิมพระชนมพรรษาสมเด็จพระบรมราชชนนีพันปีหลวง"
        years_pre = range(1914, 1976)
        years_ix = range(1976, 2017)
        years_x = range(2017, 2050)

        self.assertHolidayName(name_ix, (f"{year}-08-12" for year in years_ix))
        self.assertHolidayName(name_x, (f"{year}-08-12" for year in years_x))
        self.assertNoHolidayName(name_ix, years_pre, years_x)
        self.assertNoHolidayName(name_x, years_pre, years_ix)

        self.assertNoNonObservedHoliday(
            "2012-08-13",
            "2017-08-14",
            "2018-08-13",
            "2023-08-14",
            "2028-08-14",
            "2029-08-13",
        )

    def test_national_mothers_day(self):
        name = "วันแม่แห่งชาติ"

        self.assertHolidayName(name, (f"{year}-04-15" for year in range(1950, 1958)))
        self.assertHolidayName(name, (f"{year}-08-12" for year in range(1976, 2050)))
        self.assertNoHolidayName(name, range(1914, 1950), range(1958, 1976))

        # April 15 (1950-1958) exists prior to in lieu laws
        # In lieus are same as HM Queen Sirikit's Birthday

    def test_peace_proclamation_day(self):
        name = "วันประกาศสันติภาพ"
        self.assertHolidayName(name, (f"{year}-08-16" for year in range(1946, 1948)))
        self.assertNoHolidayName(name, range(1914, 1946), range(1948, 2050))

    def test_rama_ix_memorial_day(self):
        name_ix = "วันคล้ายวันสวรรคตพระบาทสมเด็จพระปรมินทรมหาภูมิพลอดุลยเดช บรมนาถบพิตร"
        name_x = "วันคล้ายวันสวรรคตพระบาทสมเด็จพระบรมชนกาธิเบศร มหาภูมิพลอดุลยเดชมหาราช บรมนาถบพิตร"
        name_x_memorial = "วันนวมินทรมหาราช"
        years_pre = range(1914, 2017)
        years_ix = range(2017, 2019)
        years_x = range(2019, 2023)
        years_x_memorial = range(2023, 2050)

        self.assertHolidayName(name_ix, (f"{year}-10-13" for year in years_ix))
        self.assertHolidayName(name_x, (f"{year}-10-13" for year in years_x))
        self.assertHolidayName(name_x_memorial, (f"{year}-10-13" for year in years_x_memorial))
        self.assertNoHolidayName(name_ix, years_pre, years_x, years_x_memorial)
        self.assertNoHolidayName(name_x, years_pre, years_ix, years_x_memorial)
        self.assertNoHolidayName(name_x_memorial, years_pre, years_ix, years_x)

        self.assertNoNonObservedHoliday(
            "2018-10-15",
            "2019-10-14",
            "2024-10-14",
            "2029-10-15",
            "2030-10-14",
        )

    def test_rama_five_memorial_day(self):
        name_ashes = "ทำบุญพระบรมอัษฐิพระพุทธเจ้าหลวง"
        name_death = "วันสวรรคตแห่งพระบาทสมเด็จพระพุทธเจ้าหลวง"
        name_memorial = "วันปิยมหาราช"
        years_ashes = range(1914, 1926)
        years_death = range(1926, 1938)
        years_none = range(1938, 1946)
        years_memorial = range(1946, 2050)

        self.assertHolidayName(name_ashes, (f"{year}-10-23" for year in years_ashes))
        self.assertHolidayName(name_death, (f"{year}-10-23" for year in years_death))
        self.assertHolidayName(name_memorial, (f"{year}-10-23" for year in years_memorial))
        self.assertNoHolidayName(name_ashes, years_death, years_none, years_memorial)
        self.assertNoHolidayName(name_death, years_ashes, years_none, years_memorial)
        self.assertNoHolidayName(name_memorial, years_ashes, years_death, years_none)

        self.assertNoNonObservedHoliday(
            "2010-10-25",
            "2011-10-24",
            "2016-10-24",
            "2021-10-25",
            "2022-10-24",
            "2027-10-25",
        )

    def test_united_nations_day(self):
        name = "วันสหประชาชาติ"
        self.assertHolidayName(name, (f"{year}-10-24" for year in range(1951, 1957)))
        self.assertNoHolidayName(name, range(1914, 1951), range(1957, 2050))

    def test_rama_vi_to_ix_birthday(self):
        name_1914 = "เฉลิมพระชนมพรรษา"
        name_1926 = "เฉลิมพระชนม์พรรษา"
        name_1938 = "วันเฉลิมพระชนม์พรรษา"
        name_1941 = "วันเกิดในสมเด็จพระเจ้าอยู่หัว"
        name_1945 = "วันเฉลิมพระชนมพรรษา"
        name_1960 = "วันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระปรมินทรมหาภูมิพลอดุลยเดช บรมนาถบพิตร"
        name_2016 = "วันคล้ายวันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระปรมินทรมหาภูมิพลอดุลยเดช บรมนาถบพิตร"
        name_2019 = (
            "วันคล้ายวันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระบรมชนกาธิเบศร มหาภูมิพลอดุลยเดชมหาราช บรมนาถบพิตร"
        )

        # Rama VI.
        for year in range(1914, 1925):
            self.assertHolidayName(name_1914, (f"{year}-12-30", f"{year}-12-31"))
        for year in range(1915, 1926):
            self.assertHolidayName(name_1914, (f"{year}-01-01", f"{year}-01-02", f"{year}-01-03"))
        # Rama VII.
        for year in range(1926, 1935):
            self.assertHolidayName(name_1926, (f"{year}-11-07", f"{year}-11-08", f"{year}-11-09"))
        # Rama VIII.
        self.assertHolidayName(name_1938, (f"{year}-09-20" for year in range(1938, 1940)))
        self.assertHolidayName(name_1938, "1940-09-20", "1940-09-21")
        for year in range(1941, 1945):
            self.assertHolidayName(name_1941, (f"{year}-09-20", f"{year}-09-21"))
        self.assertHolidayName(name_1945, "1945-09-20", "1945-09-21")
        # Rama IX.
        for year in range(1946, 1948):
            self.assertHolidayName(name_1945, (f"{year}-12-05", f"{year}-12-06"))
        for year in [*range(1948, 1951), *range(1952, 1954)]:
            self.assertHolidayName(name_1945, (f"{year}-12-04", f"{year}-12-05", f"{year}-12-06"))
        self.assertHolidayName(name_1945, "1951-12-05", "1951-12-06", "1951-12-07")
        self.assertHolidayName(name_1960, (f"{year}-12-05" for year in range(1960, 2016)))
        self.assertHolidayName(name_2016, (f"{year}-12-05" for year in range(2016, 2019)))
        self.assertHolidayName(name_2019, (f"{year}-12-05" for year in range(2019, 2050)))
        self.assertNoHolidayName(name_1914, range(1926, 2050))
        self.assertNoHolidayName(name_1926, range(1914, 1926), range(1935, 2050))
        self.assertNoHolidayName(name_1938, range(1914, 1938), range(1941, 2050))
        self.assertNoHolidayName(name_1941, range(1914, 1941), range(1945, 2050))
        self.assertNoHolidayName(name_1945, range(1914, 1945), range(1960, 2050))
        self.assertNoHolidayName(name_1960, range(1914, 1960), range(2016, 2050))
        self.assertNoHolidayName(name_2016, range(1914, 2016), range(2019, 2050))
        self.assertNoHolidayName(name_2019, range(1914, 2019))

        self.assertNoNonObservedHoliday(
            "2010-12-06",
            "2015-12-07",
            "2020-12-07",
            "2021-12-06",
            "2026-12-07",
            "2027-12-06",
        )

    def test_national_fathers_day(self):
        name = "วันพ่อแห่งชาติ"

        # This concides with HM King Bhumibol Adulyadej's Birthday
        self.assertHolidayName(name, (f"{year}-12-05" for year in range(1980, 2050)))
        self.assertNoHolidayName(name, range(1914, 1980))

        # In lieus are same as HM King Bhumibol Adulyadej's Birthday

    def test_constitution_day(self):
        name = "วันรัฐธรรมนูญ"

        for year in [*range(1938, 1948), *range(1950, 1954)]:
            self.assertHolidayName(name, (f"{year}-12-09", f"{year}-12-10", f"{year}-12-11"))
        self.assertHolidayName(
            name,
            (f"{year}-12-10" for year in range(1948, 1950)),
            (f"{year}-12-10" for year in range(1954, 2050)),
        )
        self.assertNoHolidayName(name, range(1914, 1938))

        self.assertNoNonObservedHoliday(
            "2011-12-12",
            "2016-12-12",
            "2017-12-11",
            "2022-12-12",
            "2023-12-11",
            "2028-12-11",
        )

    def test_new_years_eve(self):
        name = "วันสิ้นปี"

        self.assertHolidayName(
            name,
            (f"{year}-12-31" for year in range(1941, 1957)),
            (f"{year}-12-31" for year in range(1989, 2050)),
        )
        self.assertNoHolidayName(name, range(1914, 1941), range(1957, 1989))

        self.assertNoNonObservedHoliday(
            "2012-01-03",
            "2017-01-03",
            "2018-01-02",
            "2023-01-03",
            "2029-01-02",
        )

    def test_makha_bucha(self):
        self.assertNoHolidayName("มาฆบูชา จาตุรงฅ์สันนิบาต", 1914, range(1938, 2050))
        self.assertNoHolidayName("วันมาฆบูชา", range(1914, 1938))

        self.assertNoNonObservedHoliday(
            "2010-03-01",
            "2017-02-13",
            "2020-02-10",
            "2024-02-26",
            "2024-02-22",
            "2030-02-18",
        )

    def test_visakha_bucha(self):
        self.assertNoHolidayName("วิสาขะบูชา", range(1926, 2050))
        self.assertNoHolidayName("วิศาขะบูชา", range(1914, 1926), range(1938, 2050))
        self.assertNoHolidayName("วันวิสาขะบูชา", range(1914, 1938), range(1957, 2050))
        self.assertNoHolidayName("วันวิสาขบูชา", range(1914, 1957))

        self.assertNoNonObservedHoliday(
            "2019-05-20",
            "2022-05-16",
            "2023-06-05",
            "2025-05-12",
            "2026-06-01",
            "2029-05-28",
        )

    def test_asarnha_bucha(self):
        self.assertNoHolidayName("วันอาสาฬหบูชา", range(1914, 1962))

        self.assertNoNonObservedHoliday(
            "2017-07-10",
            "2020-07-07",
            "2021-07-26",
            "2024-07-22",
            "2027-07-20",
            "2030-07-16",
        )

    def test_buddhist_lent_day(self):
        self.assertNoHolidayName("เข้าปุริมพรรษา", range(1938, 2050))
        self.assertNoHolidayName("วันเข้าพรรษา", range(1914, 1938))

        self.assertNoNonObservedHoliday(
            "2011-07-18",
            "2014-07-14",
            "2018-07-30",
        )

    def test_raeknakhwan(self):
        name = "วันพืชมงคล"

        dt = (
            "1960-05-02",
            "1961-05-11",
            "1962-05-07",
            "1963-05-10",
            "1964-05-08",
            "1965-05-13",
            "1966-05-13",
            "1967-05-11",
            "1968-05-10",
            "1969-05-09",
            "1970-05-08",
            "1971-05-07",
            "1972-05-08",
            "1973-05-07",
            "1974-05-08",
            "1975-05-07",
            "1976-05-10",
            "1977-05-12",
            "1978-05-11",
            "1979-05-07",
            "1980-05-14",
            "1981-05-07",
            "1982-05-19",
            "1983-05-11",
            "1984-05-10",
            "1985-05-09",
            "1986-05-09",
            "1987-05-08",
            "1988-05-11",
            "1989-05-11",
            "1990-05-11",
            "1991-05-10",
            "1992-05-14",
            "1993-05-17",
            "1994-05-11",
            "1995-05-10",
            "1996-05-16",
            "1997-05-09",
            "1998-05-08",
            # Not a holiday in 1999 date, was held on MAY, 14.
            "2000-05-15",
            "2001-05-16",
            "2002-05-09",
            "2003-05-08",
            "2004-05-07",
            "2005-05-11",
            "2006-05-11",
            "2007-05-10",
            "2008-05-09",
            "2009-05-11",
            "2010-05-13",
            "2011-05-13",
            "2012-05-09",
            "2013-05-13",
            "2014-05-09",
            "2015-05-13",
            "2016-05-09",
            "2017-05-12",
            "2018-05-14",
            "2019-05-09",
            "2020-05-11",
            "2021-05-10",
            "2022-05-13",
            "2023-05-17",
            "2024-05-10",
            "2025-05-09",
        )
        gov_holidays = Thailand(categories=GOVERNMENT, years=range(1959, 2026))
        self.assertHolidayName(name, gov_holidays, dt)
        self.assertNoHolidayName(name, gov_holidays, 1959, 1999)
        self.assertNoHolidayName(name)

    def test_armed_forces_holiday(self):
        name = "วันกองทัพไทย"

        armed_forces_holidays = Thailand(categories=ARMED_FORCES, years=range(1958, 2050))
        self.assertHolidayName(
            name, armed_forces_holidays, (f"{year}-04-08" for year in range(1959, 1980))
        )
        self.assertHolidayName(
            name, armed_forces_holidays, (f"{year}-01-25" for year in range(1980, 2007))
        )
        self.assertHolidayName(
            name, armed_forces_holidays, (f"{year}-01-18" for year in range(2007, 2050))
        )
        self.assertNoHolidayName(name, armed_forces_holidays, 1958)
        self.assertNoHolidayName(name)

    def test_bank_holiday(self):
        name_agri = "วันหยุดเพิ่มเติมสำหรับการปิดบัญชีประจำปีของธนาคารเพื่อการเกษตรและสหกรณ์การเกษตร"
        name_mid = "วันหยุดภาคครึ่งปีของสถาบันการเงินและสถาบันการเงินเฉพาะกิจ"

        bank_holidays = Thailand(categories=BANK, years=range(1942, 2023))
        self.assertHolidayName(
            name_agri, bank_holidays, (f"{year}-04-01" for year in range(1943, 2022))
        )
        self.assertHolidayName(
            name_mid, bank_holidays, (f"{year}-07-01" for year in range(1943, 2019))
        )
        self.assertNoHolidayName(name_agri, bank_holidays, 1942, range(2022, 2050))
        self.assertNoHolidayName(name_mid, bank_holidays, 1942, range(2019, 2050))
        self.assertNoHolidayName(name_agri)
        self.assertNoHolidayName(name_mid)

    def test_school_holiday(self):
        name = "วันครู"

        school_holidays = Thailand(categories=SCHOOL, years=range(1956, 2050))
        self.assertHolidayName(
            name, school_holidays, (f"{year}-01-16" for year in range(1957, 2050))
        )
        self.assertNoHolidayName(name, school_holidays, 1956)
        self.assertNoHolidayName(name)

    def test_workday_1947(self):
        self.assertHolidays(
            Thailand(categories=WORKDAY, years=1947),
            ("1947-11-27", "วันลอยกระทง"),
        )

    def test_workday_1948(self):
        self.assertHolidays(
            Thailand(categories=WORKDAY, years=1948),
            ("1948-02-03", "วันทหารผ่านศึก"),
            ("1948-11-15", "วันลอยกระทง"),
        )

    def test_workday_1982(self):
        self.assertHolidays(
            Thailand(categories=WORKDAY, years=1982),
            ("1982-02-03", "วันทหารผ่านศึก"),
            ("1982-08-18", "วันวิทยาศาสตร์แห่งชาติ"),
            ("1982-10-31", "วันลอยกระทง"),
        )

    def test_workday_1985(self):
        self.assertHolidays(
            Thailand(categories=WORKDAY, years=1985),
            ("1985-02-03", "วันทหารผ่านศึก"),
            ("1985-02-26", "วันศิลปินแห่งชาติ"),
            ("1985-08-18", "วันวิทยาศาสตร์แห่งชาติ"),
            ("1985-11-26", "วันลอยกระทง"),
        )

    def test_workday_1989(self):
        self.assertHolidays(
            Thailand(categories=WORKDAY, years=1989),
            ("1989-02-03", "วันทหารผ่านศึก"),
            ("1989-02-26", "วันศิลปินแห่งชาติ"),
            ("1989-03-08", "วันสตรีสากล"),
            ("1989-08-18", "วันวิทยาศาสตร์แห่งชาติ"),
            ("1989-11-12", "วันลอยกระทง"),
        )

    def test_workday_1990(self):
        self.assertHolidays(
            Thailand(categories=WORKDAY, years=1990),
            ("1990-01-14", "วันอนุรักษ์ทรัพยากรป่าไม้ของชาติ"),
            ("1990-01-17", "วันพ่อขุนรามคำแหงมหาราช"),
            ("1990-02-03", "วันทหารผ่านศึก"),
            ("1990-02-26", "วันศิลปินแห่งชาติ"),
            ("1990-03-08", "วันสตรีสากล"),
            ("1990-08-18", "วันวิทยาศาสตร์แห่งชาติ"),
            ("1990-11-02", "วันลอยกระทง"),
        )

    def test_workday_1995(self):
        self.assertHolidays(
            Thailand(categories=WORKDAY, years=1995),
            ("1995-01-13", "วันการบินแห่งชาติ"),
            ("1995-01-14", "วันอนุรักษ์ทรัพยากรป่าไม้ของชาติ"),
            ("1995-01-17", "วันพ่อขุนรามคำแหงมหาราช"),
            ("1995-02-03", "วันทหารผ่านศึก"),
            ("1995-02-26", "วันศิลปินแห่งชาติ"),
            ("1995-03-08", "วันสตรีสากล"),
            ("1995-08-18", "วันวิทยาศาสตร์แห่งชาติ"),
            ("1995-11-06", "วันลอยกระทง"),
        )

    def test_workday_1999(self):
        self.assertHolidays(
            Thailand(categories=WORKDAY, years=1999),
            ("1999-01-13", "วันการบินแห่งชาติ"),
            ("1999-01-14", "วันอนุรักษ์ทรัพยากรป่าไม้ของชาติ"),
            ("1999-01-17", "วันพ่อขุนรามคำแหงมหาราช"),
            ("1999-02-03", "วันทหารผ่านศึก"),
            ("1999-02-26", "วันศิลปินแห่งชาติ"),
            ("1999-03-08", "วันสตรีสากล"),
            ("1999-05-14", "วันพืชมงคล"),
            ("1999-08-18", "วันวิทยาศาสตร์แห่งชาติ"),
            ("1999-11-22", "วันลอยกระทง"),
        )

    def test_workday_2017(self):
        self.assertHolidays(
            Thailand(categories=WORKDAY, years=2017),
            ("2017-01-13", "วันการบินแห่งชาติ"),
            ("2017-01-14", "วันอนุรักษ์ทรัพยากรป่าไม้ของชาติ"),
            ("2017-01-17", "วันพ่อขุนรามคำแหงมหาราช"),
            ("2017-02-03", "วันทหารผ่านศึก"),
            ("2017-02-26", "วันศิลปินแห่งชาติ"),
            ("2017-03-08", "วันสตรีสากล"),
            ("2017-08-18", "วันวิทยาศาสตร์แห่งชาติ"),
            ("2017-09-28", "วันพระราชทานธงชาติไทย"),
            ("2017-11-03", "วันลอยกระทง"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-03", "ชดเชยวันขึ้นปีใหม่"),
            ("2022-01-08", "วันเด็กแห่งชาติ"),
            ("2022-01-13", "วันการบินแห่งชาติ"),
            ("2022-01-14", "วันอนุรักษ์ทรัพยากรป่าไม้ของชาติ"),
            ("2022-01-16", "วันครู"),
            ("2022-01-17", "วันพ่อขุนรามคำแหงมหาราช"),
            ("2022-01-18", "วันกองทัพไทย"),
            ("2022-02-03", "วันทหารผ่านศึก"),
            ("2022-02-16", "วันมาฆบูชา"),
            ("2022-02-26", "วันศิลปินแห่งชาติ"),
            ("2022-03-08", "วันสตรีสากล"),
            ("2022-04-06", "วันพระบาทสมเด็จพระพุทธยอดฟ้าจุฬาโลกมหาราช และวันที่ระลึกมหาจักรีบรมราชวงศ์"),
            ("2022-04-13", "วันสงกรานต์"),
            ("2022-04-14", "วันสงกรานต์"),
            ("2022-04-15", "วันสงกรานต์"),
            ("2022-05-01", "วันแรงงานแห่งชาติ"),
            ("2022-05-02", "ชดเชยวันแรงงานแห่งชาติ"),
            ("2022-05-04", "วันฉัตรมงคล"),
            ("2022-05-13", "วันพืชมงคล"),
            ("2022-05-15", "วันวิสาขบูชา"),
            ("2022-05-16", "ชดเชยวันวิสาขบูชา"),
            (
                "2022-06-03",
                "วันเฉลิมพระชนมพรรษาสมเด็จพระนางเจ้าสุทิดา พัชรสุธาพิมลลักษณ พระบรมราชินี",
            ),
            ("2022-07-13", "วันอาสาฬหบูชา"),
            ("2022-07-14", "วันเข้าพรรษา"),
            ("2022-07-15", "วันหยุดพิเศษ (เพิ่มเติม)"),
            (
                "2022-07-28",
                (
                    "วันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระปรเมนทรรามาธิบดี"
                    "ศรีสินทรมหาวชิราลงกรณ พระวชิรเกล้าเจ้าอยู่หัว"
                ),
            ),
            ("2022-07-29", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2022-08-12", "วันเฉลิมพระชนมพรรษาสมเด็จพระบรมราชชนนีพันปีหลวง; วันแม่แห่งชาติ"),
            ("2022-08-18", "วันวิทยาศาสตร์แห่งชาติ"),
            ("2022-09-28", "วันพระราชทานธงชาติไทย"),
            (
                "2022-10-13",
                "วันคล้ายวันสวรรคตพระบาทสมเด็จพระบรมชนกาธิเบศร มหาภูมิพลอดุลยเดชมหาราช บรมนาถบพิตร",
            ),
            ("2022-10-14", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2022-10-23", "วันปิยมหาราช"),
            ("2022-10-24", "ชดเชยวันปิยมหาราช"),
            ("2022-11-08", "วันลอยกระทง"),
            (
                "2022-12-05",
                (
                    "วันคล้ายวันเฉลิมพระชนมพรรษาพระบาทสมเด็จพระบรม"
                    "ชนกาธิเบศร มหาภูมิพลอดุลยเดชมหาราช บรมนาถบพิตร; วันชาติ; วันพ่อแห่งชาติ"
                ),
            ),
            ("2022-12-10", "วันรัฐธรรมนูญ"),
            ("2022-12-12", "ชดเชยวันรัฐธรรมนูญ"),
            ("2022-12-30", "วันหยุดพิเศษ (เพิ่มเติม)"),
            ("2022-12-31", "วันสิ้นปี"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (in lieu)"),
            ("2022-01-08", "National Children's Day"),
            ("2022-01-13", "National Aviation Day"),
            ("2022-01-14", "National Forest Conservation Day"),
            ("2022-01-16", "Teacher's Day"),
            ("2022-01-17", "HM King Ramkamhaeng Memorial Day"),
            ("2022-01-18", "Royal Thai Armed Forces Day"),
            ("2022-02-03", "Thai Veterans Day"),
            ("2022-02-16", "Makha Bucha"),
            ("2022-02-26", "National Artist Day"),
            ("2022-03-08", "International Women's Day"),
            ("2022-04-06", "Chakri Memorial Day"),
            ("2022-04-13", "Songkran Festival"),
            ("2022-04-14", "Songkran Festival"),
            ("2022-04-15", "Songkran Festival"),
            ("2022-05-01", "National Labor Day"),
            ("2022-05-02", "National Labor Day (in lieu)"),
            ("2022-05-04", "Coronation Day"),
            ("2022-05-13", "Royal Ploughing Ceremony"),
            ("2022-05-15", "Visakha Bucha"),
            ("2022-05-16", "Visakha Bucha (in lieu)"),
            ("2022-06-03", "HM Queen Suthida's Birthday"),
            ("2022-07-13", "Asarnha Bucha"),
            ("2022-07-14", "Buddhist Lent Day"),
            ("2022-07-15", "Bridge Public Holiday"),
            ("2022-07-28", "HM King Maha Vajiralongkorn's Birthday"),
            ("2022-07-29", "Bridge Public Holiday"),
            (
                "2022-08-12",
                "HM Queen Sirikit The Queen Mother's Birthday; National Mother's Day",
            ),
            ("2022-08-18", "National Science Day"),
            ("2022-09-28", "Thai National Flag Day"),
            ("2022-10-13", "HM King Bhumibol Adulyadej the Great Memorial Day"),
            ("2022-10-14", "Bridge Public Holiday"),
            ("2022-10-23", "HM King Chulalongkorn Memorial Day"),
            ("2022-10-24", "HM King Chulalongkorn Memorial Day (in lieu)"),
            ("2022-11-08", "Loy Krathong"),
            (
                "2022-12-05",
                (
                    "HM King Bhumibol Adulyadej the Great's Birthday; "
                    "National Day; National Father's Day"
                ),
            ),
            ("2022-12-10", "Constitution Day"),
            ("2022-12-12", "Constitution Day (in lieu)"),
            ("2022-12-30", "Bridge Public Holiday"),
            ("2022-12-31", "New Year's Eve"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.latvia import Latvia, LV, LVA
from tests.common import CommonCountryTests


class TestLatvia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Latvia, years=range(1990, 2050))

    def test_country_aliases(self):
        self.assertAliases(Latvia, LV, LVA)

    def test_no_holidays(self):
        self.assertNoHolidays(Latvia(years=1989))

    def test_special_holidays(self):
        self.assertHoliday(
            "2018-07-09",
            "2018-09-24",
            "2023-05-29",
            "2023-07-10",
        )

    def test_new_years(self):
        self.assertHolidayName("Jaunais Gads", (f"{year}-01-01" for year in range(1990, 2050)))

    def test_good_friday(self):
        self.assertHolidayName(
            "Lielā Piektdiena",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )

    def test_easter(self):
        self.assertHolidayName(
            "Lieldienas",
            "2019-04-21",
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
        )

    def test_easter_monday(self):
        self.assertHolidayName(
            "Otrās Lieldienas",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
        )

    def test_labor_day(self):
        self.assertHolidayName("Darba svētki", (f"{year}-05-01" for year in range(1990, 2050)))

    def test_restoration_of_independence_day(self):
        name = "Latvijas Republikas Neatkarības atjaunošanas diena"
        self.assertHolidayName(name, (f"{year}-05-04" for year in range(2002, 2050)))
        self.assertNoHoliday(f"{year}-05-04" for year in range(1990, 2002))
        self.assertNoHolidayName(name, range(1990, 2002))

        dt = (
            "2008-05-05",
            "2013-05-06",
            "2014-05-05",
            "2019-05-06",
        )
        self.assertHolidayName(f"{name} (brīvdiena)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_mothers_day(self):
        self.assertHolidayName(
            "Mātes diena",
            "2019-05-12",
            "2020-05-10",
            "2021-05-09",
            "2022-05-08",
            "2023-05-14",
        )

    def test_midsummer_eve(self):
        self.assertHolidayName("Līgo diena", (f"{year}-06-23" for year in range(1990, 2050)))

    def test_midsummer_day(self):
        self.assertHolidayName("Jāņu diena", (f"{year}-06-24" for year in range(1990, 2050)))

    def test_republic_proclamation_day(self):
        name = "Latvijas Republikas proklamēšanas diena"
        self.assertHolidayName(name, (f"{year}-11-18" for year in range(1990, 2050)))

        dt = (
            "2007-11-19",
            "2012-11-19",
            "2017-11-20",
            "2018-11-19",
            "2023-11-20",
        )
        self.assertHolidayName(f"{name} (brīvdiena)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_christmas_eve(self):
        name = "Ziemassvētku vakars"
        self.assertHolidayName(name, (f"{year}-12-24" for year in range(2007, 2050)))
        self.assertNoHoliday(f"{year}-12-24" for year in range(1990, 2007))
        self.assertNoHolidayName(name, range(1990, 2007))

    def test_christmas_day(self):
        self.assertHolidayName("Ziemassvētki", (f"{year}-12-25" for year in range(1990, 2050)))

    def test_second_christmas_day(self):
        self.assertHolidayName(
            "Otrie Ziemassvētki", (f"{year}-12-26" for year in range(1990, 2050))
        )

    def test_new_years_eve(self):
        self.assertHolidayName("Vecgada vakars", (f"{year}-12-31" for year in range(1990, 2050)))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Jaunais Gads"),
            ("2022-04-15", "Lielā Piektdiena"),
            ("2022-04-17", "Lieldienas"),
            ("2022-04-18", "Otrās Lieldienas"),
            ("2022-05-01", "Darba svētki"),
            ("2022-05-04", "Latvijas Republikas Neatkarības atjaunošanas diena"),
            ("2022-05-08", "Mātes diena"),
            ("2022-06-23", "Līgo diena"),
            ("2022-06-24", "Jāņu diena"),
            ("2022-11-18", "Latvijas Republikas proklamēšanas diena"),
            ("2022-12-24", "Ziemassvētku vakars"),
            ("2022-12-25", "Ziemassvētki"),
            ("2022-12-26", "Otrie Ziemassvētki"),
            ("2022-12-31", "Vecgada vakars"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-04", "Restoration of Independence Day"),
            ("2022-05-08", "Mother's Day"),
            ("2022-06-23", "Midsummer Eve"),
            ("2022-06-24", "Midsummer Day"),
            ("2022-11-18", "Republic of Latvia Proclamation Day"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "День праці"),
            ("2022-05-04", "День відновлення незалежности Латвійської Республіки"),
            ("2022-05-08", "День матері"),
            ("2022-06-23", "Ліго"),
            ("2022-06-24", "Янів день"),
            ("2022-11-18", "День проголошення Латвійської Республіки"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
            ("2022-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.cameroon import Cameroon, CM, CMR
from tests.common import CommonCountryTests


class TestCameroon(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Cameroon, years=range(1960, 2050))

    def test_country_aliases(self):
        self.assertAliases(Cameroon, CM, CMR)

    def test_no_holidays(self):
        self.assertNoHolidays(Cameroon(years=1959))

    def test_special_holidays(self):
        self.assertHoliday("2021-05-14", "2021-07-19")

    def test_youth_day(self):
        name = "Youth Day"
        self.assertHolidayName(name, (f"{year}-02-11" for year in range(1966, 2050)))
        self.assertNoHolidayName(name, range(1960, 1966))
        self.assertNoHoliday(f"{year}-02-11" for year in range(1960, 1966))

    def test_national_day(self):
        name = "National Day"
        self.assertHolidayName(name, (f"{year}-05-20" for year in range(1972, 2050)))
        self.assertNoHolidayName(name, range(1960, 1972))
        self.assertNoHoliday(f"{year}-05-20" for year in set(range(1960, 1972)).difference({1971}))

    def test_observed(self):
        dt = (
            # New Year's Day
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
            # Youth Day
            "2018-02-12",
            "2024-02-12",
            # Labour Day
            "2016-05-02",
            "2022-05-03",
            # National Day
            "2012-05-21",
            "2018-05-21",
            # Assumption Day
            "2010-08-16",
            "2021-08-16",
            # Christmas Day
            "2011-12-26",
            "2016-12-26",
            "2022-12-26",
            # Eid al-Fitr
            "2012-08-20",
            "2020-05-25",
            # Eid al-Adha
            "2007-01-02",  # special case
            "2014-10-06",
            "2019-08-12",
            # Mawlid
            "2012-02-06",
            "2019-11-11",
        )
        self.assertHoliday(dt)
        self.assertNoNonObservedHoliday(dt)

    def test_2022(self):
        self.assertHolidays(
            Cameroon(years=2022),
            ("2022-01-01", "New Year's Day"),
            ("2022-02-11", "Youth Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-05-01", "Labour Day"),
            ("2022-05-02", "Eid al-Fitr"),
            ("2022-05-03", "Labour Day (observed)"),
            ("2022-05-20", "National Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-08-15", "Assumption Day"),
            ("2022-10-08", "Mawlid"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.gabon import Gabon, GA, GAB
from tests.common import CommonCountryTests


class TestGabon(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Gabon)

    def test_country_aliases(self):
        self.assertAliases(Gabon, GA, GAB)

    def test_no_holidays(self):
        self.assertNoHolidays(Gabon(years=1960))

    def test_womens_rights_day(self):
        name = "Women's Rights Day"
        self.assertHolidayName(name, (f"{year}-04-17" for year in range(2015, 2050)))
        self.assertNoHolidayName(name, Gabon(years=range(1961, 2015)))
        self.assertNoHoliday(
            f"{year}-04-17" for year in set(range(1961, 2015)).difference({1995, 1997, 2006})
        )

    def test_2022(self):
        self.assertHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-04-17", "Women's Rights Day"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labour Day"),
            ("2022-05-02", "Eid al-Fitr"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-06", "Whit Monday"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-08-15", "Assumption Day"),
            ("2022-08-16", "Independence Day"),
            ("2022-08-17", "Independence Day Holiday"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.ethiopia import Ethiopia, ET, ETH
from tests.common import CommonCountryTests


class TestEthiopia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Ethiopia, years=range(1940, 2050))

    def test_country_aliases(self):
        self.assertAliases(Ethiopia, ET, ETH)

    def test_no_holidays(self):
        self.assertNoHolidays(Ethiopia(years=1896))

    def test_christmas(self):
        self.assertHolidayName("ገና", (f"{year}-01-07" for year in range(1940, 2050)))

    def test_ephiphany(self):
        name = "ጥምቀት"
        self.assertHolidayName(
            name, (f"{year}-01-19" for year in range(1940, 2050) if year % 4 != 0)
        )
        self.assertHolidayName(
            name, (f"{year}-01-20" for year in range(1940, 2050) if year % 4 == 0)
        )

    def test_adwa_victory(self):
        self.assertHolidayName("አድዋ", (f"{year}-03-02" for year in range(1940, 2050)))

    def test_good_friday(self):
        name = "ስቅለት"
        self.assertHolidayName(
            name,
            "2018-04-06",
            "2019-04-26",
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
        )
        self.assertHolidayName(name, range(1940, 2050))

    def test_easter(self):
        name = "ፋሲካ"
        self.assertHolidayName(
            name,
            "2018-04-08",
            "2019-04-28",
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
        )
        self.assertHolidayName(name, range(1940, 2050))

    def test_workers_day(self):
        self.assertHolidayName("የሰራተኞች ቀን", (f"{year}-05-01" for year in range(1940, 2050)))

    def test_patriots_day(self):
        name = "የአርበኞች ቀን"
        self.assertHolidayName(name, (f"{year}-05-05" for year in range(1942, 2050)))
        self.assertNoHolidayName(name, range(1940, 1942))

    def test_downfall_of_dergue(self):
        name = "ደርግ የወደቀበት ቀን"
        self.assertHolidayName(name, (f"{year}-05-28" for year in range(1992, 2050)))
        self.assertNoHolidayName(name, range(1940, 1992))

    def test_new_year(self):
        name = "እንቁጣጣሽ"
        self.assertHolidayName(
            name, (f"{year}-09-11" for year in range(1940, 2050) if year % 4 != 3)
        )
        self.assertHolidayName(
            name, (f"{year}-09-12" for year in range(1940, 2050) if year % 4 == 3)
        )

    def test_finding_of_true_cross(self):
        name = "መስቀል"
        self.assertHolidayName(
            name, (f"{year}-09-27" for year in range(1940, 2050) if year % 4 != 3)
        )
        self.assertHolidayName(
            name, (f"{year}-09-28" for year in range(1940, 2050) if year % 4 == 3)
        )

    def test_revolution_day(self):
        name = "የአብዮት ቀን"
        self.assertHolidayName(
            name, (f"{year}-09-12" for year in range(1975, 1991) if year % 4 != 3)
        )
        self.assertHolidayName(
            name, (f"{year}-09-13" for year in range(1975, 1991) if year % 4 == 3)
        )
        self.assertNoHolidayName(name, range(1940, 1975), range(1991, 2050))

    def test_october_revolution_day(self):
        name = "የጥቅምት አብዮት ቀን"
        self.assertHolidayName(name, (f"{year}-11-07" for year in range(1975, 1991)))
        self.assertNoHolidayName(name, range(1940, 1975), range(1991, 2050))

    def test_eid_al_fitr(self):
        self.assertHolidayName(
            "ኢድ አልፈጥር",
            "2018-06-15",
            "2019-06-04",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
        )

    def test_eid_al_adha(self):
        self.assertHolidayName(
            "አረፋ",
            "2018-08-22",
            "2019-08-11",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
        )

    def test_prophets_birthday(self):
        self.assertHolidayName(
            "መውሊድ",
            "2018-11-21",
            "2019-11-10",
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-07", "ገና"),
            ("2022-01-19", "ጥምቀት"),
            ("2022-03-02", "አድዋ"),
            ("2022-04-22", "ስቅለት"),
            ("2022-04-24", "ፋሲካ"),
            ("2022-05-01", "የሰራተኞች ቀን"),
            ("2022-05-02", "ኢድ አልፈጥር"),
            ("2022-05-05", "የአርበኞች ቀን"),
            ("2022-05-28", "ደርግ የወደቀበት ቀን"),
            ("2022-07-09", "አረፋ"),
            ("2022-09-11", "እንቁጣጣሽ"),
            ("2022-09-27", "መስቀል"),
            ("2022-10-08", "መውሊድ"),
        )

    def test_l10n_en_ar(self):
        self.assertLocalizedHolidays(
            "ar",
            ("2022-01-07", "عيد الميلاد"),
            ("2022-01-19", "عيد الغطاس"),
            ("2022-03-02", "العدوة"),
            ("2022-04-22", "جمعة جيدة"),
            ("2022-04-24", "عيد الفصح"),
            ("2022-05-01", "يوم العمال"),
            ("2022-05-02", "عيد الفطر"),
            ("2022-05-05", "يوم الوطنيين"),
            ("2022-05-28", "يوم سقوط ديرج"),
            ("2022-07-09", "عيد الأضحى"),
            ("2022-09-11", "السنة الإثيوبية الجديدة"),
            ("2022-09-27", "مسكل"),
            ("2022-10-08", "عيد المولد النبوي"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-07", "Christmas Day"),
            ("2022-01-19", "Epiphany Day"),
            ("2022-03-02", "Adwa Victory Day"),
            ("2022-04-22", "Good Friday"),
            ("2022-04-24", "Easter Sunday"),
            ("2022-05-01", "Workers' Day"),
            ("2022-05-02", "Eid al-Fitr"),
            ("2022-05-05", "Patriots' Day"),
            ("2022-05-28", "Downfall of Dergue Regime Day"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-09-11", "Ethiopian New Year"),
            ("2022-09-27", "Finding of True Cross"),
            ("2022-10-08", "Prophet's Birthday"),
        )
